"use strict";
var meow = require("meow"), chalk = require("chalk");
function _interopDefaultCompat(e) {
  return e && typeof e == "object" && "default" in e ? e : { default: e };
}
var meow__default = /* @__PURE__ */ _interopDefaultCompat(meow), chalk__default = /* @__PURE__ */ _interopDefaultCompat(chalk);
let beQuiet = !1, beVerbose = !1;
function setVerbosity({ verbose, silent }) {
  silent ? (beVerbose = !1, beQuiet = !0) : verbose && (beVerbose = !0, beQuiet = !1);
}
var log = {
  setVerbosity,
  // Bypasses any checks, prints regardless (only use for things like `cli --version`)
  msg: (msg, ...args) => !beQuiet && console.log(msg, ...args),
  // Debug only printed on --verbose
  debug: (msg, ...args) => !beQuiet && beVerbose && console.debug(`${chalk__default.default.bgBlack.white("[debug]")} ${msg}`, ...args),
  // Success messages only printed if not --silent
  success: (msg, ...args) => !beQuiet && console.info(`${chalk__default.default.bgBlack.greenBright("[success]")} ${msg}`, ...args),
  // Info only printed if not --silent ("standard" level)
  info: (msg, ...args) => !beQuiet && console.info(`${chalk__default.default.bgBlack.cyanBright("[info]")} ${msg}`, ...args),
  // Warning only printed if not --silent
  warn: (msg, ...args) => !beQuiet && console.warn(`${chalk__default.default.bgBlack.yellowBright("[warn]")} ${msg}`, ...args),
  // Errors are always printed
  error: (msg, ...args) => console.error(`${chalk__default.default.bgBlack.redBright("[error]")} ${msg}`, ...args)
}, commands = {
  init: async (options) => {
    await (await Promise.resolve().then(function() {
      return require("./init2.js");
    })).default(options);
  },
  inject: async (options) => {
    await (await Promise.resolve().then(function() {
      return require("./inject.js");
    })).default(options);
  },
  "link-watch": async (options) => {
    await (await Promise.resolve().then(function() {
      return require("./link-watch.js");
    })).default(options);
  },
  "verify-package": async (options) => {
    await (await Promise.resolve().then(function() {
      return require("./verify-package.js");
    })).default(options);
  },
  "verify-studio": async (options) => {
    await (await Promise.resolve().then(function() {
      return require("./verify-studio.js");
    })).default(options);
  },
  version: async (options) => {
    await (await Promise.resolve().then(function() {
      return require("./version.js");
    })).default(options);
  }
}, sharedFlags = {
  debug: {
    default: !1,
    type: "boolean"
  },
  silent: {
    type: "boolean",
    default: !1
  },
  verbose: {
    type: "boolean",
    default: !1
  }
};
const cliName = "@sanity/plugin-kit", urls = {
  refDocs: "https://beta.sanity.io/docs/reference",
  migrationGuideStudio: "https://beta.sanity.io/docs/platform/v2-to-v3",
  migrationGuidePlugin: "https://beta.sanity.io/docs/platform/v2-to-v3/plugins",
  pluginReadme: "https://github.com/sanity-io/plugin-kit",
  incompatiblePlugin: "https://github.com/sanity-io/incompatible-plugin",
  sanityExchange: "https://www.sanity.io/exchange",
  linterPackage: "https://github.com/sanity-io/eslint-config-no-v2-imports"
}, incompatiblePluginPackage = "@sanity/incompatible-plugin", defaultOutDir = "dist";
async function cliEntry(argv = process.argv, autoExit = !0) {
  const cli = meow__default.default(
    `
	Usage
	  $ ${cliName} [--help] [--debug] <command> [<args>]

  These are common commands used in various situations:

    init            Create a new Sanity plugin
    inject          Inject config into an existing Sanity v3 plugin
    verify-package  Check that a Sanity plugin package follows V3 conventions. Prints upgrade steps.
    verify-studio   Check that a Sanity Studio follows V3 conventions. Prints upgrade steps.
    link-watch      Recompiles plugin automatically on changes and runs yalc push --publish
    version         Show the version of ${cliName} currently installed

  Options
    --silent        Do not print info and warning messages
    --verbose       Log everything. This option conflicts with --silent
    --debug         Print stack trace on errors
    --version       Output the version number
    --help          Output usage information

  Examples
    # Init a new plugin in current directory
    $ ${cliName} init

    # Init a new plugin in my-sanity-plugin directory
    $ ${cliName} init my-sanity-plugin

    # Check that a Sanity plugin package in current directory follows V3 conventions
    $ ${cliName} verify-package

    # Check that a Sanity Studio in current directory  follows V3 conventions
    $ ${cliName} verify-studio
`,
    {
      autoHelp: !1,
      flags: sharedFlags,
      argv: argv.slice(2)
    }
  ), commandName = cli.input[0];
  commandName || cli.showHelp(), commandName in commands || (console.error(`Unknown command "${commandName}"`), cli.showHelp()), cli.flags.silent && cli.flags.verbose && (log.error("--silent and --verbose are mutually exclusive"), cli.showHelp());
  const cmd = commands[commandName];
  try {
    log.setVerbosity(cli.flags), await cmd({ argv: argv.slice(3) });
  } catch (err) {
    log.error(err instanceof TypeError || cli.flags.debug ? err.stack : err.message), process.exit(1);
  }
}
exports.cliEntry = cliEntry;
exports.cliName = cliName;
exports.defaultOutDir = defaultOutDir;
exports.incompatiblePluginPackage = incompatiblePluginPackage;
exports.log = log;
exports.sharedFlags = sharedFlags;
exports.urls = urls;
//# sourceMappingURL=cli.js.map
