"use strict";
var path = require("path"), meow = require("meow"), cli = require("./cli.js"), init = require("./init.js"), _package = require("./package.js"), execa = require("execa");
function _interopDefaultCompat(e) {
  return e && typeof e == "object" && "default" in e ? e : { default: e };
}
var path__default = /* @__PURE__ */ _interopDefaultCompat(path), meow__default = /* @__PURE__ */ _interopDefaultCompat(meow), execa__default = /* @__PURE__ */ _interopDefaultCompat(execa);
function npmIsAvailable() {
  return execa__default.default("npm", ["-v"]).then(() => !0).catch(() => !1);
}
function yarnIsAvailable() {
  return execa__default.default("yarn", ["-v"]).then(() => !0).catch(() => !1);
}
function pnpmAvailable() {
  return execa__default.default("pnpm", ["-v"]).then(() => !0).catch(() => !1);
}
async function promptForPackageManager() {
  const [npm, yarn, pnpm] = await Promise.all([
    npmIsAvailable(),
    yarnIsAvailable(),
    pnpmAvailable()
  ]), choices = [npm && "npm", yarn && "yarn", pnpm && "pnpm"].filter(Boolean);
  return choices.length < 2 ? choices[0] || "npm" : _package.prompt("Which package manager do you prefer?", {
    choices: choices.map((value) => ({ value, name: value })),
    default: choices[0]
  });
}
async function installDependencies(pm, { cwd }) {
  const proc = execa__default.default(pm, ["install"], { cwd, stdio: "inherit" }), { exitCode } = await proc;
  return exitCode <= 0;
}
const description = "Initialize a new Sanity plugin", help = `
Usage
  $ ${cli.cliName} init [dir] [<args>]

Options
  --no-eslint             Disables ESLint config and dependencies from being added
  --no-prettier           Disables prettier config and dependencies from being added
  --no-typescript         Disables typescript config and dependencies from being added
  --no-license            Disables LICENSE + package.json license field from being added
  --no-editorconfig       Disables .editorconfig from being added
  --no-gitignore          Disables .gitignore from being added
  --no-scripts            Disables scripts from being added to package.json
  --no-install            Disables automatically running package manager install

  --name [package-name]   Use the provided package-name
  --author [name]         Use the provided author
  --repo [url]            Use the provided repo url
  --license [spdx]        Use the license with the given SPDX identifier
  --force                 No promt when overwriting files

  --preset [preset-name]  Adds config and files from a named preset. --preset can be supplied multiple times.
                          The following presets are available:
${init.presetHelpList(30)}

Examples
  # Initialize a new plugin in the current directory
  $ ${cli.cliName} init

  # Initialize a plugin in the directory ~/my-plugin
  $ ${cli.cliName} init ~/my-plugin

  # Don't add eslint or prettier
  $ ${cli.cliName} init --no-eslint --no-prettier
`;
async function run({ argv }) {
  const cli$1 = meow__default.default(help, { flags: init.initFlags, argv, description }), basePath = path__default.default.resolve(cli$1.input[0] || process.cwd()), { exists, isRoot } = await _package.hasSanityJson(basePath);
  if (exists && isRoot)
    throw new Error(
      'sanity.json has a "root" property set to true - are you trying to init into a studio instead of a plugin?'
    );
  const { v3ConfigFile } = await _package.findStudioV3Config(basePath);
  if (v3ConfigFile)
    throw new Error(
      `${v3ConfigFile} exsists - are you trying to init into a studio instead of a plugin?`
    );
  if (cli.log.info('Initializing new plugin in "%s"', basePath), !cli$1.flags.force && !await _package.isEmptyish(basePath) && !await _package.prompt("Directory is not empty, proceed?", { type: "confirm", default: !1 })) {
    cli.log.error("Directory is not empty. Cancelled.");
    return;
  }
  await _package.ensureDir(basePath), await init.init({ basePath, flags: cli$1.flags }), cli$1.flags.install ? await installDependencies(await promptForPackageManager(), { cwd: basePath }) ? cli.log.info("Done!") : cli.log.error("Failed to install dependencies, try manually running `npm install`") : cli.log.info("Dependency installation skipped.");
}
exports.default = run;
//# sourceMappingURL=init2.js.map
