"use strict";
var pkgUtils = require("@sanity/pkg-utils"), path = require("path"), meow = require("meow"), cli = require("./cli.js"), init = require("./init.js"), _package = require("./package.js");
function _interopDefaultCompat(e) {
  return e && typeof e == "object" && "default" in e ? e : { default: e };
}
var path__default = /* @__PURE__ */ _interopDefaultCompat(path), meow__default = /* @__PURE__ */ _interopDefaultCompat(meow);
const description = "Inject configuration into a Sanity plugin", help = `
Usage
  $ ${cli.cliName} inject [dir] [<args>]

Options
  --no-eslint             Disables ESLint config and dependencies from being added
  --no-prettier           Disables prettier config and dependencies from being added
  --no-typescript         Disables typescript config and dependencies from being added
  --no-license            Disables LICENSE + package.json license field from being added
  --no-editorconfig       Disables .editorconfig from being added
  --no-gitignore          Disables .gitignore from being added
  --no-scripts            Disables scripts from being added to package.json

  --license [spdx]        Use the license with the given SPDX identifier
  --force                 No promt when overwriting files

  --preset [preset-name]  Adds config and files from a named preset. --preset can be supplied multiple times.
                          The following presets are available:
${init.presetHelpList(30)}
  --preset-only           Skips the default inject steps. Use this to apply a preset to an otherwise complete plugin.

Examples
  # Inject configuration into the plugin in the current directory
  $ ${cli.cliName} inject

  # Inject configuration into the plugin in ~/my-plugin
  $ ${cli.cliName} inject ~/my-plugin

  # Don't inject eslint or prettier
  $ ${cli.cliName} inject --no-eslint --no-prettier

  # Inject plugin configuration and semver-workflow into the plugin in the current directory
  $ @sanity/plugin-kit inject --preset semver-workflow

  # Only inject semver-workflow and renovatebot config from presets
  $ ${cli.cliName} inject --preset-only --preset semver-workflow --preset renovatebot

`;
async function run({ argv }) {
  const cli$1 = meow__default.default(help, { flags: init.initFlags, argv, description }), basePath = path__default.default.resolve(cli$1.input[0] || process.cwd()), outDir = (await pkgUtils.loadConfig({ cwd: basePath }))?.dist ?? cli.defaultOutDir, { v3ConfigFile } = await _package.findStudioV3Config(basePath);
  if (v3ConfigFile)
    throw new Error(
      `${v3ConfigFile} exists - are you trying to INJECT into a studio instead of a plugin?`
    );
  cli.log.info('Inject config into plugin in "%s"', basePath), await init.inject({ basePath, outDir, flags: cli$1.flags, validate: !1 }), cli.log.info("Done!");
}
exports.default = run;
//# sourceMappingURL=inject.js.map
