"use strict";
var path = require("path"), meow = require("meow"), _package$1 = require("./package2.js"), cli = require("./cli.js"), nodemon = require("nodemon"), concurrently = require("concurrently"), chalk = require("chalk"), fs = require("fs"), _package = require("./package.js"), outdent = require("outdent"), pkgUtils = require("@sanity/pkg-utils");
function _interopDefaultCompat(e) {
  return e && typeof e == "object" && "default" in e ? e : { default: e };
}
var path__default = /* @__PURE__ */ _interopDefaultCompat(path), meow__default = /* @__PURE__ */ _interopDefaultCompat(meow), nodemon__default = /* @__PURE__ */ _interopDefaultCompat(nodemon), concurrently__default = /* @__PURE__ */ _interopDefaultCompat(concurrently), chalk__default = /* @__PURE__ */ _interopDefaultCompat(chalk), fs__default = /* @__PURE__ */ _interopDefaultCompat(fs), outdent__default = /* @__PURE__ */ _interopDefaultCompat(outdent);
async function linkWatch({ basePath }) {
  const packageJson = JSON.parse(
    fs__default.default.readFileSync(path__default.default.join(basePath, "package.json"), "utf8")
  ), outDir = (await pkgUtils.loadConfig({ cwd: basePath }))?.dist ?? cli.defaultOutDir, watch = {
    command: "npm run watch",
    extensions: "ts,js,png,svg,gif,jpeg,css",
    ...packageJson.sanityPlugin?.linkWatch
  };
  nodemon__default.default({
    watch: [outDir],
    ext: watch.extensions,
    exec: "yalc push --changed"
    //delay: 1000
  });
  const folder = path__default.default.join(basePath, outDir);
  await _package.fileExists(folder) || await _package.mkdir(folder);
  const pkg = await _package.getPackage({ basePath, validate: !1 });
  concurrently__default.default([watch.command]), nodemon__default.default.on("start", function() {
    cli.log.info(
      outdent__default.default`
        Watching ${outDir} for changes to files with extensions: ${watch.extensions}

        To test this package in Sanity Studio or another package, in a separate shell run:
        cd /path/to/sanity/studio-or-package

        Then, run one of the below commands, based on the package manager used in studio-or-package:

        ## yarn
        ${chalk__default.default.greenBright(`yalc add --link ${pkg.name} && yarn install`)}

        ## npm
        ${chalk__default.default.greenBright(`npx yalc add ${pkg.name} && npx yalc link ${pkg.name} && npm install`)}
      `.trimStart()
    );
  }).on("quit", function() {
    process.exit();
  }).on("restart", function(files) {
    cli.log.info("Found changes in files:", chalk__default.default.magentaBright(files)), cli.log.info("Pushing new yalc package...");
  });
}
const description = "Run the watch command and pushes any changes to yalc", help = `
Usage
  $ ${_package$1.pkg.binname} link-watch [<args>]

Options
  --silent      Do not print info and warning messages
  --verbose     Log everything. This option conflicts with --silent
  --version     Output the version number
  --help        Output usage information

Configuration
To override the default watch command configuration, provide an override in package.json under sanityPlugin:
{
  "sanityPlugin": {
    "watchCommand": "microbundle watch --format modern,esm,cjs --jsx React.createElement --jsxImportSource react --css inline",
    "linkWatch": {
      "command": "npm run watch",
      "extensions": "js,png,svg,gif,jpeg,css"
    }
  }
}

Examples
  # Run the watch command and pushes any changes to yalc
  $ ${_package$1.pkg.binname} link-watch
`, flags = {
  ...cli.sharedFlags,
  watch: {
    type: "boolean",
    default: !1
  }
};
function run({ argv }) {
  const cli2 = meow__default.default(help, { flags, argv, description }), basePath = path__default.default.resolve(cli2.input[0] || process.cwd());
  return linkWatch({ basePath });
}
exports.default = run;
//# sourceMappingURL=link-watch.js.map
