"use strict";
var fs = require("fs"), path = require("path"), util$1 = require("util"), githubUrlToObject = require("github-url-to-object"), validateNpmPackageName = require("validate-npm-package-name"), _package = require("./package2.js"), pAny = require("p-any"), crypto = require("crypto"), url = require("url"), inquirer = require("inquirer"), cli = require("./cli.js"), pProps = require("p-props"), getLatestVersion = require("get-latest-version"), outdent = require("outdent"), chalk = require("chalk");
function _interopDefaultCompat(e) {
  return e && typeof e == "object" && "default" in e ? e : { default: e };
}
var fs__default = /* @__PURE__ */ _interopDefaultCompat(fs), path__default = /* @__PURE__ */ _interopDefaultCompat(path), util__default = /* @__PURE__ */ _interopDefaultCompat(util$1), githubUrlToObject__default = /* @__PURE__ */ _interopDefaultCompat(githubUrlToObject), validateNpmPackageName__default = /* @__PURE__ */ _interopDefaultCompat(validateNpmPackageName), pAny__default = /* @__PURE__ */ _interopDefaultCompat(pAny), crypto__default = /* @__PURE__ */ _interopDefaultCompat(crypto), inquirer__default = /* @__PURE__ */ _interopDefaultCompat(inquirer), pProps__default = /* @__PURE__ */ _interopDefaultCompat(pProps), getLatestVersion__default = /* @__PURE__ */ _interopDefaultCompat(getLatestVersion), outdent__default = /* @__PURE__ */ _interopDefaultCompat(outdent), chalk__default = /* @__PURE__ */ _interopDefaultCompat(chalk);
const buildExtensions = [".js", ".jsx", ".es6", ".es", ".mjs", ".ts", ".tsx"];
async function prompt(message, options) {
  const type = options.choices ? "list" : options.type, result = await inquirer__default.default.prompt([{ ...options, type, message, name: "single" }]);
  return result && result.single;
}
prompt.separator = () => new inquirer__default.default.Separator();
function promptForPackageName({ basePath }, defaultVal) {
  return prompt("Plugin name (sanity-plugin-...)", {
    default: defaultVal || path__default.default.basename(basePath),
    filter: (name) => {
      const prefixless = name.trim().replace(/^sanity-plugin-/, "");
      return name[0] === "@" ? name : `sanity-plugin-${prefixless}`;
    },
    validate: (name) => {
      const valid = validateNpmPackageName__default.default(name);
      return valid.errors ? valid.errors[0] : name[0] !== "@" && name.endsWith("plugin") ? `Name shouldn't include "plugin" multiple times (${name})` : !0;
    }
  });
}
function promptForRepoOrigin(options, defaultVal) {
  return prompt("Git repository URL", {
    default: defaultVal,
    filter: (raw) => {
      const url2 = (raw || "").trim(), gh = githubUrlToObject__default.default(url2);
      return gh ? `git+ssh://git@github.com/${gh.user}/${gh.repo}.git` : url2;
    },
    validate: (url$1) => {
      if (!url$1)
        return !0;
      try {
        return new url.URL(url$1) ? !0 : "Invalid URL";
      } catch {
        return "Invalid URL";
      }
    }
  });
}
var Space_Separator = /[\u1680\u2000-\u200A\u202F\u205F\u3000]/, ID_Start = /[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0370-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u05D0-\u05EA\u05F0-\u05F2\u0620-\u064A\u066E\u066F\u0671-\u06D3\u06D5\u06E5\u06E6\u06EE\u06EF\u06FA-\u06FC\u06FF\u0710\u0712-\u072F\u074D-\u07A5\u07B1\u07CA-\u07EA\u07F4\u07F5\u07FA\u0800-\u0815\u081A\u0824\u0828\u0840-\u0858\u0860-\u086A\u08A0-\u08B4\u08B6-\u08BD\u0904-\u0939\u093D\u0950\u0958-\u0961\u0971-\u0980\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BD\u09CE\u09DC\u09DD\u09DF-\u09E1\u09F0\u09F1\u09FC\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A59-\u0A5C\u0A5E\u0A72-\u0A74\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABD\u0AD0\u0AE0\u0AE1\u0AF9\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3D\u0B5C\u0B5D\u0B5F-\u0B61\u0B71\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BD0\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D\u0C58-\u0C5A\u0C60\u0C61\u0C80\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBD\u0CDE\u0CE0\u0CE1\u0CF1\u0CF2\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D\u0D4E\u0D54-\u0D56\u0D5F-\u0D61\u0D7A-\u0D7F\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0E01-\u0E30\u0E32\u0E33\u0E40-\u0E46\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB0\u0EB2\u0EB3\u0EBD\u0EC0-\u0EC4\u0EC6\u0EDC-\u0EDF\u0F00\u0F40-\u0F47\u0F49-\u0F6C\u0F88-\u0F8C\u1000-\u102A\u103F\u1050-\u1055\u105A-\u105D\u1061\u1065\u1066\u106E-\u1070\u1075-\u1081\u108E\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176C\u176E-\u1770\u1780-\u17B3\u17D7\u17DC\u1820-\u1877\u1880-\u1884\u1887-\u18A8\u18AA\u18B0-\u18F5\u1900-\u191E\u1950-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u1A00-\u1A16\u1A20-\u1A54\u1AA7\u1B05-\u1B33\u1B45-\u1B4B\u1B83-\u1BA0\u1BAE\u1BAF\u1BBA-\u1BE5\u1C00-\u1C23\u1C4D-\u1C4F\u1C5A-\u1C7D\u1C80-\u1C88\u1CE9-\u1CEC\u1CEE-\u1CF1\u1CF5\u1CF6\u1D00-\u1DBF\u1E00-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u2071\u207F\u2090-\u209C\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CEE\u2CF2\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2E2F\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303C\u3041-\u3096\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312E\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FEA\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA61F\uA62A\uA62B\uA640-\uA66E\uA67F-\uA69D\uA6A0-\uA6EF\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AE\uA7B0-\uA7B7\uA7F7-\uA801\uA803-\uA805\uA807-\uA80A\uA80C-\uA822\uA840-\uA873\uA882-\uA8B3\uA8F2-\uA8F7\uA8FB\uA8FD\uA90A-\uA925\uA930-\uA946\uA960-\uA97C\uA984-\uA9B2\uA9CF\uA9E0-\uA9E4\uA9E6-\uA9EF\uA9FA-\uA9FE\uAA00-\uAA28\uAA40-\uAA42\uAA44-\uAA4B\uAA60-\uAA76\uAA7A\uAA7E-\uAAAF\uAAB1\uAAB5\uAAB6\uAAB9-\uAABD\uAAC0\uAAC2\uAADB-\uAADD\uAAE0-\uAAEA\uAAF2-\uAAF4\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABE2\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D\uFB1F-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE70-\uFE74\uFE76-\uFEFC\uFF21-\uFF3A\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDD40-\uDD74\uDE80-\uDE9C\uDEA0-\uDED0\uDF00-\uDF1F\uDF2D-\uDF4A\uDF50-\uDF75\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF\uDFD1-\uDFD5]|\uD801[\uDC00-\uDC9D\uDCB0-\uDCD3\uDCD8-\uDCFB\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDCE0-\uDCF2\uDCF4\uDCF5\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00\uDE10-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE4\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48\uDC80-\uDCB2\uDCC0-\uDCF2]|\uD804[\uDC03-\uDC37\uDC83-\uDCAF\uDCD0-\uDCE8\uDD03-\uDD26\uDD50-\uDD72\uDD76\uDD83-\uDDB2\uDDC1-\uDDC4\uDDDA\uDDDC\uDE00-\uDE11\uDE13-\uDE2B\uDE80-\uDE86\uDE88\uDE8A-\uDE8D\uDE8F-\uDE9D\uDE9F-\uDEA8\uDEB0-\uDEDE\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3D\uDF50\uDF5D-\uDF61]|\uD805[\uDC00-\uDC34\uDC47-\uDC4A\uDC80-\uDCAF\uDCC4\uDCC5\uDCC7\uDD80-\uDDAE\uDDD8-\uDDDB\uDE00-\uDE2F\uDE44\uDE80-\uDEAA\uDF00-\uDF19]|\uD806[\uDCA0-\uDCDF\uDCFF\uDE00\uDE0B-\uDE32\uDE3A\uDE50\uDE5C-\uDE83\uDE86-\uDE89\uDEC0-\uDEF8]|\uD807[\uDC00-\uDC08\uDC0A-\uDC2E\uDC40\uDC72-\uDC8F\uDD00-\uDD06\uDD08\uDD09\uDD0B-\uDD30\uDD46]|\uD808[\uDC00-\uDF99]|\uD809[\uDC00-\uDC6E\uDC80-\uDD43]|[\uD80C\uD81C-\uD820\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872\uD874-\uD879][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD811[\uDC00-\uDE46]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDED0-\uDEED\uDF00-\uDF2F\uDF40-\uDF43\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50\uDF93-\uDF9F\uDFE0\uDFE1]|\uD821[\uDC00-\uDFEC]|\uD822[\uDC00-\uDEF2]|\uD82C[\uDC00-\uDD1E\uDD70-\uDEFB]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB]|\uD83A[\uDC00-\uDCC4\uDD00-\uDD43]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1\uDEB0-\uDFFF]|\uD87A[\uDC00-\uDFE0]|\uD87E[\uDC00-\uDE1D]/, ID_Continue = /[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0300-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u0483-\u0487\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u0591-\u05BD\u05BF\u05C1\u05C2\u05C4\u05C5\u05C7\u05D0-\u05EA\u05F0-\u05F2\u0610-\u061A\u0620-\u0669\u066E-\u06D3\u06D5-\u06DC\u06DF-\u06E8\u06EA-\u06FC\u06FF\u0710-\u074A\u074D-\u07B1\u07C0-\u07F5\u07FA\u0800-\u082D\u0840-\u085B\u0860-\u086A\u08A0-\u08B4\u08B6-\u08BD\u08D4-\u08E1\u08E3-\u0963\u0966-\u096F\u0971-\u0983\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BC-\u09C4\u09C7\u09C8\u09CB-\u09CE\u09D7\u09DC\u09DD\u09DF-\u09E3\u09E6-\u09F1\u09FC\u0A01-\u0A03\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A3C\u0A3E-\u0A42\u0A47\u0A48\u0A4B-\u0A4D\u0A51\u0A59-\u0A5C\u0A5E\u0A66-\u0A75\u0A81-\u0A83\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABC-\u0AC5\u0AC7-\u0AC9\u0ACB-\u0ACD\u0AD0\u0AE0-\u0AE3\u0AE6-\u0AEF\u0AF9-\u0AFF\u0B01-\u0B03\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3C-\u0B44\u0B47\u0B48\u0B4B-\u0B4D\u0B56\u0B57\u0B5C\u0B5D\u0B5F-\u0B63\u0B66-\u0B6F\u0B71\u0B82\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BBE-\u0BC2\u0BC6-\u0BC8\u0BCA-\u0BCD\u0BD0\u0BD7\u0BE6-\u0BEF\u0C00-\u0C03\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D-\u0C44\u0C46-\u0C48\u0C4A-\u0C4D\u0C55\u0C56\u0C58-\u0C5A\u0C60-\u0C63\u0C66-\u0C6F\u0C80-\u0C83\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBC-\u0CC4\u0CC6-\u0CC8\u0CCA-\u0CCD\u0CD5\u0CD6\u0CDE\u0CE0-\u0CE3\u0CE6-\u0CEF\u0CF1\u0CF2\u0D00-\u0D03\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D44\u0D46-\u0D48\u0D4A-\u0D4E\u0D54-\u0D57\u0D5F-\u0D63\u0D66-\u0D6F\u0D7A-\u0D7F\u0D82\u0D83\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0DCA\u0DCF-\u0DD4\u0DD6\u0DD8-\u0DDF\u0DE6-\u0DEF\u0DF2\u0DF3\u0E01-\u0E3A\u0E40-\u0E4E\u0E50-\u0E59\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB9\u0EBB-\u0EBD\u0EC0-\u0EC4\u0EC6\u0EC8-\u0ECD\u0ED0-\u0ED9\u0EDC-\u0EDF\u0F00\u0F18\u0F19\u0F20-\u0F29\u0F35\u0F37\u0F39\u0F3E-\u0F47\u0F49-\u0F6C\u0F71-\u0F84\u0F86-\u0F97\u0F99-\u0FBC\u0FC6\u1000-\u1049\u1050-\u109D\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u135D-\u135F\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1714\u1720-\u1734\u1740-\u1753\u1760-\u176C\u176E-\u1770\u1772\u1773\u1780-\u17D3\u17D7\u17DC\u17DD\u17E0-\u17E9\u180B-\u180D\u1810-\u1819\u1820-\u1877\u1880-\u18AA\u18B0-\u18F5\u1900-\u191E\u1920-\u192B\u1930-\u193B\u1946-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u19D0-\u19D9\u1A00-\u1A1B\u1A20-\u1A5E\u1A60-\u1A7C\u1A7F-\u1A89\u1A90-\u1A99\u1AA7\u1AB0-\u1ABD\u1B00-\u1B4B\u1B50-\u1B59\u1B6B-\u1B73\u1B80-\u1BF3\u1C00-\u1C37\u1C40-\u1C49\u1C4D-\u1C7D\u1C80-\u1C88\u1CD0-\u1CD2\u1CD4-\u1CF9\u1D00-\u1DF9\u1DFB-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u203F\u2040\u2054\u2071\u207F\u2090-\u209C\u20D0-\u20DC\u20E1\u20E5-\u20F0\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D7F-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2DE0-\u2DFF\u2E2F\u3005-\u3007\u3021-\u302F\u3031-\u3035\u3038-\u303C\u3041-\u3096\u3099\u309A\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312E\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FEA\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA62B\uA640-\uA66F\uA674-\uA67D\uA67F-\uA6F1\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AE\uA7B0-\uA7B7\uA7F7-\uA827\uA840-\uA873\uA880-\uA8C5\uA8D0-\uA8D9\uA8E0-\uA8F7\uA8FB\uA8FD\uA900-\uA92D\uA930-\uA953\uA960-\uA97C\uA980-\uA9C0\uA9CF-\uA9D9\uA9E0-\uA9FE\uAA00-\uAA36\uAA40-\uAA4D\uAA50-\uAA59\uAA60-\uAA76\uAA7A-\uAAC2\uAADB-\uAADD\uAAE0-\uAAEF\uAAF2-\uAAF6\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABEA\uABEC\uABED\uABF0-\uABF9\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE00-\uFE0F\uFE20-\uFE2F\uFE33\uFE34\uFE4D-\uFE4F\uFE70-\uFE74\uFE76-\uFEFC\uFF10-\uFF19\uFF21-\uFF3A\uFF3F\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDD40-\uDD74\uDDFD\uDE80-\uDE9C\uDEA0-\uDED0\uDEE0\uDF00-\uDF1F\uDF2D-\uDF4A\uDF50-\uDF7A\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF\uDFD1-\uDFD5]|\uD801[\uDC00-\uDC9D\uDCA0-\uDCA9\uDCB0-\uDCD3\uDCD8-\uDCFB\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDCE0-\uDCF2\uDCF4\uDCF5\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00-\uDE03\uDE05\uDE06\uDE0C-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE38-\uDE3A\uDE3F\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE6\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48\uDC80-\uDCB2\uDCC0-\uDCF2]|\uD804[\uDC00-\uDC46\uDC66-\uDC6F\uDC7F-\uDCBA\uDCD0-\uDCE8\uDCF0-\uDCF9\uDD00-\uDD34\uDD36-\uDD3F\uDD50-\uDD73\uDD76\uDD80-\uDDC4\uDDCA-\uDDCC\uDDD0-\uDDDA\uDDDC\uDE00-\uDE11\uDE13-\uDE37\uDE3E\uDE80-\uDE86\uDE88\uDE8A-\uDE8D\uDE8F-\uDE9D\uDE9F-\uDEA8\uDEB0-\uDEEA\uDEF0-\uDEF9\uDF00-\uDF03\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3C-\uDF44\uDF47\uDF48\uDF4B-\uDF4D\uDF50\uDF57\uDF5D-\uDF63\uDF66-\uDF6C\uDF70-\uDF74]|\uD805[\uDC00-\uDC4A\uDC50-\uDC59\uDC80-\uDCC5\uDCC7\uDCD0-\uDCD9\uDD80-\uDDB5\uDDB8-\uDDC0\uDDD8-\uDDDD\uDE00-\uDE40\uDE44\uDE50-\uDE59\uDE80-\uDEB7\uDEC0-\uDEC9\uDF00-\uDF19\uDF1D-\uDF2B\uDF30-\uDF39]|\uD806[\uDCA0-\uDCE9\uDCFF\uDE00-\uDE3E\uDE47\uDE50-\uDE83\uDE86-\uDE99\uDEC0-\uDEF8]|\uD807[\uDC00-\uDC08\uDC0A-\uDC36\uDC38-\uDC40\uDC50-\uDC59\uDC72-\uDC8F\uDC92-\uDCA7\uDCA9-\uDCB6\uDD00-\uDD06\uDD08\uDD09\uDD0B-\uDD36\uDD3A\uDD3C\uDD3D\uDD3F-\uDD47\uDD50-\uDD59]|\uD808[\uDC00-\uDF99]|\uD809[\uDC00-\uDC6E\uDC80-\uDD43]|[\uD80C\uD81C-\uD820\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872\uD874-\uD879][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD811[\uDC00-\uDE46]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDE60-\uDE69\uDED0-\uDEED\uDEF0-\uDEF4\uDF00-\uDF36\uDF40-\uDF43\uDF50-\uDF59\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50-\uDF7E\uDF8F-\uDF9F\uDFE0\uDFE1]|\uD821[\uDC00-\uDFEC]|\uD822[\uDC00-\uDEF2]|\uD82C[\uDC00-\uDD1E\uDD70-\uDEFB]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99\uDC9D\uDC9E]|\uD834[\uDD65-\uDD69\uDD6D-\uDD72\uDD7B-\uDD82\uDD85-\uDD8B\uDDAA-\uDDAD\uDE42-\uDE44]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB\uDFCE-\uDFFF]|\uD836[\uDE00-\uDE36\uDE3B-\uDE6C\uDE75\uDE84\uDE9B-\uDE9F\uDEA1-\uDEAF]|\uD838[\uDC00-\uDC06\uDC08-\uDC18\uDC1B-\uDC21\uDC23\uDC24\uDC26-\uDC2A]|\uD83A[\uDC00-\uDCC4\uDCD0-\uDCD6\uDD00-\uDD4A\uDD50-\uDD59]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1\uDEB0-\uDFFF]|\uD87A[\uDC00-\uDFE0]|\uD87E[\uDC00-\uDE1D]|\uDB40[\uDD00-\uDDEF]/, unicode = {
  Space_Separator,
  ID_Start,
  ID_Continue
}, util = {
  isSpaceSeparator(c2) {
    return typeof c2 == "string" && unicode.Space_Separator.test(c2);
  },
  isIdStartChar(c2) {
    return typeof c2 == "string" && (c2 >= "a" && c2 <= "z" || c2 >= "A" && c2 <= "Z" || c2 === "$" || c2 === "_" || unicode.ID_Start.test(c2));
  },
  isIdContinueChar(c2) {
    return typeof c2 == "string" && (c2 >= "a" && c2 <= "z" || c2 >= "A" && c2 <= "Z" || c2 >= "0" && c2 <= "9" || c2 === "$" || c2 === "_" || c2 === "\u200C" || c2 === "\u200D" || unicode.ID_Continue.test(c2));
  },
  isDigit(c2) {
    return typeof c2 == "string" && /[0-9]/.test(c2);
  },
  isHexDigit(c2) {
    return typeof c2 == "string" && /[0-9A-Fa-f]/.test(c2);
  }
};
let source, parseState, stack, pos, line, column, token, key, root;
var parse = function(text, reviver) {
  source = String(text), parseState = "start", stack = [], pos = 0, line = 1, column = 0, token = void 0, key = void 0, root = void 0;
  do
    token = lex(), parseStates[parseState]();
  while (token.type !== "eof");
  return typeof reviver == "function" ? internalize({ "": root }, "", reviver) : root;
};
function internalize(holder, name, reviver) {
  const value = holder[name];
  if (value != null && typeof value == "object")
    if (Array.isArray(value))
      for (let i = 0; i < value.length; i++) {
        const key2 = String(i), replacement = internalize(value, key2, reviver);
        replacement === void 0 ? delete value[key2] : Object.defineProperty(value, key2, {
          value: replacement,
          writable: !0,
          enumerable: !0,
          configurable: !0
        });
      }
    else
      for (const key2 in value) {
        const replacement = internalize(value, key2, reviver);
        replacement === void 0 ? delete value[key2] : Object.defineProperty(value, key2, {
          value: replacement,
          writable: !0,
          enumerable: !0,
          configurable: !0
        });
      }
  return reviver.call(holder, name, value);
}
let lexState, buffer, doubleQuote, sign, c;
function lex() {
  for (lexState = "default", buffer = "", doubleQuote = !1, sign = 1; ; ) {
    c = peek();
    const token2 = lexStates[lexState]();
    if (token2)
      return token2;
  }
}
function peek() {
  if (source[pos])
    return String.fromCodePoint(source.codePointAt(pos));
}
function read() {
  const c2 = peek();
  return c2 === `
` ? (line++, column = 0) : c2 ? column += c2.length : column++, c2 && (pos += c2.length), c2;
}
const lexStates = {
  default() {
    switch (c) {
      case "	":
      case "\v":
      case "\f":
      case " ":
      case "\xA0":
      case "\uFEFF":
      case `
`:
      case "\r":
      case "\u2028":
      case "\u2029":
        read();
        return;
      case "/":
        read(), lexState = "comment";
        return;
      case void 0:
        return read(), newToken("eof");
    }
    if (util.isSpaceSeparator(c)) {
      read();
      return;
    }
    return lexStates[parseState]();
  },
  comment() {
    switch (c) {
      case "*":
        read(), lexState = "multiLineComment";
        return;
      case "/":
        read(), lexState = "singleLineComment";
        return;
    }
    throw invalidChar(read());
  },
  multiLineComment() {
    switch (c) {
      case "*":
        read(), lexState = "multiLineCommentAsterisk";
        return;
      case void 0:
        throw invalidChar(read());
    }
    read();
  },
  multiLineCommentAsterisk() {
    switch (c) {
      case "*":
        read();
        return;
      case "/":
        read(), lexState = "default";
        return;
      case void 0:
        throw invalidChar(read());
    }
    read(), lexState = "multiLineComment";
  },
  singleLineComment() {
    switch (c) {
      case `
`:
      case "\r":
      case "\u2028":
      case "\u2029":
        read(), lexState = "default";
        return;
      case void 0:
        return read(), newToken("eof");
    }
    read();
  },
  value() {
    switch (c) {
      case "{":
      case "[":
        return newToken("punctuator", read());
      case "n":
        return read(), literal("ull"), newToken("null", null);
      case "t":
        return read(), literal("rue"), newToken("boolean", !0);
      case "f":
        return read(), literal("alse"), newToken("boolean", !1);
      case "-":
      case "+":
        read() === "-" && (sign = -1), lexState = "sign";
        return;
      case ".":
        buffer = read(), lexState = "decimalPointLeading";
        return;
      case "0":
        buffer = read(), lexState = "zero";
        return;
      case "1":
      case "2":
      case "3":
      case "4":
      case "5":
      case "6":
      case "7":
      case "8":
      case "9":
        buffer = read(), lexState = "decimalInteger";
        return;
      case "I":
        return read(), literal("nfinity"), newToken("numeric", 1 / 0);
      case "N":
        return read(), literal("aN"), newToken("numeric", NaN);
      case '"':
      case "'":
        doubleQuote = read() === '"', buffer = "", lexState = "string";
        return;
    }
    throw invalidChar(read());
  },
  identifierNameStartEscape() {
    if (c !== "u")
      throw invalidChar(read());
    read();
    const u = unicodeEscape();
    switch (u) {
      case "$":
      case "_":
        break;
      default:
        if (!util.isIdStartChar(u))
          throw invalidIdentifier();
        break;
    }
    buffer += u, lexState = "identifierName";
  },
  identifierName() {
    switch (c) {
      case "$":
      case "_":
      case "\u200C":
      case "\u200D":
        buffer += read();
        return;
      case "\\":
        read(), lexState = "identifierNameEscape";
        return;
    }
    if (util.isIdContinueChar(c)) {
      buffer += read();
      return;
    }
    return newToken("identifier", buffer);
  },
  identifierNameEscape() {
    if (c !== "u")
      throw invalidChar(read());
    read();
    const u = unicodeEscape();
    switch (u) {
      case "$":
      case "_":
      case "\u200C":
      case "\u200D":
        break;
      default:
        if (!util.isIdContinueChar(u))
          throw invalidIdentifier();
        break;
    }
    buffer += u, lexState = "identifierName";
  },
  sign() {
    switch (c) {
      case ".":
        buffer = read(), lexState = "decimalPointLeading";
        return;
      case "0":
        buffer = read(), lexState = "zero";
        return;
      case "1":
      case "2":
      case "3":
      case "4":
      case "5":
      case "6":
      case "7":
      case "8":
      case "9":
        buffer = read(), lexState = "decimalInteger";
        return;
      case "I":
        return read(), literal("nfinity"), newToken("numeric", sign * (1 / 0));
      case "N":
        return read(), literal("aN"), newToken("numeric", NaN);
    }
    throw invalidChar(read());
  },
  zero() {
    switch (c) {
      case ".":
        buffer += read(), lexState = "decimalPoint";
        return;
      case "e":
      case "E":
        buffer += read(), lexState = "decimalExponent";
        return;
      case "x":
      case "X":
        buffer += read(), lexState = "hexadecimal";
        return;
    }
    return newToken("numeric", sign * 0);
  },
  decimalInteger() {
    switch (c) {
      case ".":
        buffer += read(), lexState = "decimalPoint";
        return;
      case "e":
      case "E":
        buffer += read(), lexState = "decimalExponent";
        return;
    }
    if (util.isDigit(c)) {
      buffer += read();
      return;
    }
    return newToken("numeric", sign * Number(buffer));
  },
  decimalPointLeading() {
    if (util.isDigit(c)) {
      buffer += read(), lexState = "decimalFraction";
      return;
    }
    throw invalidChar(read());
  },
  decimalPoint() {
    switch (c) {
      case "e":
      case "E":
        buffer += read(), lexState = "decimalExponent";
        return;
    }
    if (util.isDigit(c)) {
      buffer += read(), lexState = "decimalFraction";
      return;
    }
    return newToken("numeric", sign * Number(buffer));
  },
  decimalFraction() {
    switch (c) {
      case "e":
      case "E":
        buffer += read(), lexState = "decimalExponent";
        return;
    }
    if (util.isDigit(c)) {
      buffer += read();
      return;
    }
    return newToken("numeric", sign * Number(buffer));
  },
  decimalExponent() {
    switch (c) {
      case "+":
      case "-":
        buffer += read(), lexState = "decimalExponentSign";
        return;
    }
    if (util.isDigit(c)) {
      buffer += read(), lexState = "decimalExponentInteger";
      return;
    }
    throw invalidChar(read());
  },
  decimalExponentSign() {
    if (util.isDigit(c)) {
      buffer += read(), lexState = "decimalExponentInteger";
      return;
    }
    throw invalidChar(read());
  },
  decimalExponentInteger() {
    if (util.isDigit(c)) {
      buffer += read();
      return;
    }
    return newToken("numeric", sign * Number(buffer));
  },
  hexadecimal() {
    if (util.isHexDigit(c)) {
      buffer += read(), lexState = "hexadecimalInteger";
      return;
    }
    throw invalidChar(read());
  },
  hexadecimalInteger() {
    if (util.isHexDigit(c)) {
      buffer += read();
      return;
    }
    return newToken("numeric", sign * Number(buffer));
  },
  string() {
    switch (c) {
      case "\\":
        read(), buffer += escape();
        return;
      case '"':
        if (doubleQuote)
          return read(), newToken("string", buffer);
        buffer += read();
        return;
      case "'":
        if (!doubleQuote)
          return read(), newToken("string", buffer);
        buffer += read();
        return;
      case `
`:
      case "\r":
        throw invalidChar(read());
      case "\u2028":
      case "\u2029":
        separatorChar(c);
        break;
      case void 0:
        throw invalidChar(read());
    }
    buffer += read();
  },
  start() {
    switch (c) {
      case "{":
      case "[":
        return newToken("punctuator", read());
    }
    lexState = "value";
  },
  beforePropertyName() {
    switch (c) {
      case "$":
      case "_":
        buffer = read(), lexState = "identifierName";
        return;
      case "\\":
        read(), lexState = "identifierNameStartEscape";
        return;
      case "}":
        return newToken("punctuator", read());
      case '"':
      case "'":
        doubleQuote = read() === '"', lexState = "string";
        return;
    }
    if (util.isIdStartChar(c)) {
      buffer += read(), lexState = "identifierName";
      return;
    }
    throw invalidChar(read());
  },
  afterPropertyName() {
    if (c === ":")
      return newToken("punctuator", read());
    throw invalidChar(read());
  },
  beforePropertyValue() {
    lexState = "value";
  },
  afterPropertyValue() {
    switch (c) {
      case ",":
      case "}":
        return newToken("punctuator", read());
    }
    throw invalidChar(read());
  },
  beforeArrayValue() {
    if (c === "]")
      return newToken("punctuator", read());
    lexState = "value";
  },
  afterArrayValue() {
    switch (c) {
      case ",":
      case "]":
        return newToken("punctuator", read());
    }
    throw invalidChar(read());
  },
  end() {
    throw invalidChar(read());
  }
};
function newToken(type, value) {
  return {
    type,
    value,
    line,
    column
  };
}
function literal(s) {
  for (const c2 of s) {
    if (peek() !== c2)
      throw invalidChar(read());
    read();
  }
}
function escape() {
  switch (peek()) {
    case "b":
      return read(), "\b";
    case "f":
      return read(), "\f";
    case "n":
      return read(), `
`;
    case "r":
      return read(), "\r";
    case "t":
      return read(), "	";
    case "v":
      return read(), "\v";
    case "0":
      if (read(), util.isDigit(peek()))
        throw invalidChar(read());
      return "\0";
    case "x":
      return read(), hexEscape();
    case "u":
      return read(), unicodeEscape();
    case `
`:
    case "\u2028":
    case "\u2029":
      return read(), "";
    case "\r":
      return read(), peek() === `
` && read(), "";
    case "1":
    case "2":
    case "3":
    case "4":
    case "5":
    case "6":
    case "7":
    case "8":
    case "9":
      throw invalidChar(read());
    case void 0:
      throw invalidChar(read());
  }
  return read();
}
function hexEscape() {
  let buffer2 = "", c2 = peek();
  if (!util.isHexDigit(c2) || (buffer2 += read(), c2 = peek(), !util.isHexDigit(c2)))
    throw invalidChar(read());
  return buffer2 += read(), String.fromCodePoint(parseInt(buffer2, 16));
}
function unicodeEscape() {
  let buffer2 = "", count = 4;
  for (; count-- > 0; ) {
    const c2 = peek();
    if (!util.isHexDigit(c2))
      throw invalidChar(read());
    buffer2 += read();
  }
  return String.fromCodePoint(parseInt(buffer2, 16));
}
const parseStates = {
  start() {
    if (token.type === "eof")
      throw invalidEOF();
    push();
  },
  beforePropertyName() {
    switch (token.type) {
      case "identifier":
      case "string":
        key = token.value, parseState = "afterPropertyName";
        return;
      case "punctuator":
        pop();
        return;
      case "eof":
        throw invalidEOF();
    }
  },
  afterPropertyName() {
    if (token.type === "eof")
      throw invalidEOF();
    parseState = "beforePropertyValue";
  },
  beforePropertyValue() {
    if (token.type === "eof")
      throw invalidEOF();
    push();
  },
  beforeArrayValue() {
    if (token.type === "eof")
      throw invalidEOF();
    if (token.type === "punctuator" && token.value === "]") {
      pop();
      return;
    }
    push();
  },
  afterPropertyValue() {
    if (token.type === "eof")
      throw invalidEOF();
    switch (token.value) {
      case ",":
        parseState = "beforePropertyName";
        return;
      case "}":
        pop();
    }
  },
  afterArrayValue() {
    if (token.type === "eof")
      throw invalidEOF();
    switch (token.value) {
      case ",":
        parseState = "beforeArrayValue";
        return;
      case "]":
        pop();
    }
  },
  end() {
  }
};
function push() {
  let value;
  switch (token.type) {
    case "punctuator":
      switch (token.value) {
        case "{":
          value = {};
          break;
        case "[":
          value = [];
          break;
      }
      break;
    case "null":
    case "boolean":
    case "numeric":
    case "string":
      value = token.value;
      break;
  }
  if (root === void 0)
    root = value;
  else {
    const parent = stack[stack.length - 1];
    Array.isArray(parent) ? parent.push(value) : Object.defineProperty(parent, key, {
      value,
      writable: !0,
      enumerable: !0,
      configurable: !0
    });
  }
  if (value !== null && typeof value == "object")
    stack.push(value), Array.isArray(value) ? parseState = "beforeArrayValue" : parseState = "beforePropertyName";
  else {
    const current = stack[stack.length - 1];
    current == null ? parseState = "end" : Array.isArray(current) ? parseState = "afterArrayValue" : parseState = "afterPropertyValue";
  }
}
function pop() {
  stack.pop();
  const current = stack[stack.length - 1];
  current == null ? parseState = "end" : Array.isArray(current) ? parseState = "afterArrayValue" : parseState = "afterPropertyValue";
}
function invalidChar(c2) {
  return syntaxError(c2 === void 0 ? `JSON5: invalid end of input at ${line}:${column}` : `JSON5: invalid character '${formatChar(c2)}' at ${line}:${column}`);
}
function invalidEOF() {
  return syntaxError(`JSON5: invalid end of input at ${line}:${column}`);
}
function invalidIdentifier() {
  return column -= 5, syntaxError(`JSON5: invalid identifier character at ${line}:${column}`);
}
function separatorChar(c2) {
  console.warn(`JSON5: '${formatChar(c2)}' in strings is not valid ECMAScript; consider escaping`);
}
function formatChar(c2) {
  const replacements = {
    "'": "\\'",
    '"': '\\"',
    "\\": "\\\\",
    "\b": "\\b",
    "\f": "\\f",
    "\n": "\\n",
    "\r": "\\r",
    "	": "\\t",
    "\v": "\\v",
    "\0": "\\0",
    "\u2028": "\\u2028",
    "\u2029": "\\u2029"
  };
  if (replacements[c2])
    return replacements[c2];
  if (c2 < " ") {
    const hexString = c2.charCodeAt(0).toString(16);
    return "\\x" + ("00" + hexString).substring(hexString.length);
  }
  return c2;
}
function syntaxError(message) {
  const err = new SyntaxError(message);
  return err.lineNumber = line, err.columnNumber = column, err;
}
var stringify = function(value, replacer, space) {
  const stack2 = [];
  let indent = "", propertyList, replacerFunc, gap = "", quote;
  if (replacer != null && typeof replacer == "object" && !Array.isArray(replacer) && (space = replacer.space, quote = replacer.quote, replacer = replacer.replacer), typeof replacer == "function")
    replacerFunc = replacer;
  else if (Array.isArray(replacer)) {
    propertyList = [];
    for (const v of replacer) {
      let item;
      typeof v == "string" ? item = v : (typeof v == "number" || v instanceof String || v instanceof Number) && (item = String(v)), item !== void 0 && propertyList.indexOf(item) < 0 && propertyList.push(item);
    }
  }
  return space instanceof Number ? space = Number(space) : space instanceof String && (space = String(space)), typeof space == "number" ? space > 0 && (space = Math.min(10, Math.floor(space)), gap = "          ".substr(0, space)) : typeof space == "string" && (gap = space.substr(0, 10)), serializeProperty("", { "": value });
  function serializeProperty(key2, holder) {
    let value2 = holder[key2];
    switch (value2 != null && (typeof value2.toJSON5 == "function" ? value2 = value2.toJSON5(key2) : typeof value2.toJSON == "function" && (value2 = value2.toJSON(key2))), replacerFunc && (value2 = replacerFunc.call(holder, key2, value2)), value2 instanceof Number ? value2 = Number(value2) : value2 instanceof String ? value2 = String(value2) : value2 instanceof Boolean && (value2 = value2.valueOf()), value2) {
      case null:
        return "null";
      case !0:
        return "true";
      case !1:
        return "false";
    }
    if (typeof value2 == "string")
      return quoteString(value2);
    if (typeof value2 == "number")
      return String(value2);
    if (typeof value2 == "object")
      return Array.isArray(value2) ? serializeArray(value2) : serializeObject(value2);
  }
  function quoteString(value2) {
    const quotes = {
      "'": 0.1,
      '"': 0.2
    }, replacements = {
      "'": "\\'",
      '"': '\\"',
      "\\": "\\\\",
      "\b": "\\b",
      "\f": "\\f",
      "\n": "\\n",
      "\r": "\\r",
      "	": "\\t",
      "\v": "\\v",
      "\0": "\\0",
      "\u2028": "\\u2028",
      "\u2029": "\\u2029"
    };
    let product = "";
    for (let i = 0; i < value2.length; i++) {
      const c2 = value2[i];
      switch (c2) {
        case "'":
        case '"':
          quotes[c2]++, product += c2;
          continue;
        case "\0":
          if (util.isDigit(value2[i + 1])) {
            product += "\\x00";
            continue;
          }
      }
      if (replacements[c2]) {
        product += replacements[c2];
        continue;
      }
      if (c2 < " ") {
        let hexString = c2.charCodeAt(0).toString(16);
        product += "\\x" + ("00" + hexString).substring(hexString.length);
        continue;
      }
      product += c2;
    }
    const quoteChar = quote || Object.keys(quotes).reduce((a, b) => quotes[a] < quotes[b] ? a : b);
    return product = product.replace(new RegExp(quoteChar, "g"), replacements[quoteChar]), quoteChar + product + quoteChar;
  }
  function serializeObject(value2) {
    if (stack2.indexOf(value2) >= 0)
      throw TypeError("Converting circular structure to JSON5");
    stack2.push(value2);
    let stepback = indent;
    indent = indent + gap;
    let keys = propertyList || Object.keys(value2), partial = [];
    for (const key2 of keys) {
      const propertyString = serializeProperty(key2, value2);
      if (propertyString !== void 0) {
        let member = serializeKey(key2) + ":";
        gap !== "" && (member += " "), member += propertyString, partial.push(member);
      }
    }
    let final;
    if (partial.length === 0)
      final = "{}";
    else {
      let properties;
      if (gap === "")
        properties = partial.join(","), final = "{" + properties + "}";
      else {
        let separator = `,
` + indent;
        properties = partial.join(separator), final = `{
` + indent + properties + `,
` + stepback + "}";
      }
    }
    return stack2.pop(), indent = stepback, final;
  }
  function serializeKey(key2) {
    if (key2.length === 0)
      return quoteString(key2);
    const firstChar = String.fromCodePoint(key2.codePointAt(0));
    if (!util.isIdStartChar(firstChar))
      return quoteString(key2);
    for (let i = firstChar.length; i < key2.length; i++)
      if (!util.isIdContinueChar(String.fromCodePoint(key2.codePointAt(i))))
        return quoteString(key2);
    return key2;
  }
  function serializeArray(value2) {
    if (stack2.indexOf(value2) >= 0)
      throw TypeError("Converting circular structure to JSON5");
    stack2.push(value2);
    let stepback = indent;
    indent = indent + gap;
    let partial = [];
    for (let i = 0; i < value2.length; i++) {
      const propertyString = serializeProperty(String(i), value2);
      partial.push(propertyString !== void 0 ? propertyString : "null");
    }
    let final;
    if (partial.length === 0)
      final = "[]";
    else if (gap === "")
      final = "[" + partial.join(",") + "]";
    else {
      let separator = `,
` + indent, properties = partial.join(separator);
      final = `[
` + indent + properties + `,
` + stepback + "]";
    }
    return stack2.pop(), indent = stepback, final;
  }
};
const JSON5 = {
  parse,
  stringify
};
var lib = JSON5;
const stat$1 = util__default.default.promisify(fs__default.default.stat), mkdir = util__default.default.promisify(fs__default.default.mkdir), readdir = util__default.default.promisify(fs__default.default.readdir), copyFile = util__default.default.promisify(fs__default.default.copyFile), readFile$2 = util__default.default.promisify(fs__default.default.readFile), writeFile = util__default.default.promisify(fs__default.default.writeFile);
function hasSourceEquivalent(compiledFile, paths) {
  if (!paths.source)
    return fileExists(
      path__default.default.isAbsolute(compiledFile) ? compiledFile : path__default.default.resolve(paths.basePath, compiledFile)
    );
  const baseDir = path__default.default.dirname(compiledFile.replace(paths.compiled, paths.source)), baseName = path__default.default.basename(compiledFile, path__default.default.extname(compiledFile)), pathStub = path__default.default.join(baseDir, baseName);
  return buildCandidateExists(pathStub);
}
async function hasSourceFile(filePath, paths) {
  if (!paths?.source)
    return fileExists(
      path__default.default.isAbsolute(filePath) ? filePath : path__default.default.resolve(paths?.basePath ?? "", filePath)
    );
  const pathStub = path__default.default.isAbsolute(filePath) ? filePath : path__default.default.resolve(paths.source, filePath);
  return await fileExists(pathStub) ? !0 : buildCandidateExists(pathStub);
}
function hasCompiledFile(filePath, paths) {
  if (!paths?.compiled)
    return fileExists(
      path__default.default.isAbsolute(filePath) ? filePath : path__default.default.resolve(paths?.basePath ?? "", filePath)
    );
  const absPath = path__default.default.isAbsolute(filePath) ? filePath : path__default.default.resolve(paths.compiled, filePath), withExt = path__default.default.extname(absPath) === "" ? `${absPath}.js` : absPath;
  return fileExists(withExt);
}
function buildCandidateExists(pathStub) {
  const candidates = buildExtensions.map((extCandidate) => `${pathStub}${extCandidate}`);
  return pAny__default.default(candidates.map((candidate) => stat$1(candidate))).then(() => !0).catch(() => !1);
}
function fileExists(filePath) {
  return stat$1(filePath).then(() => !0).catch(() => !1);
}
async function readJsonFile(filePath) {
  const content = await readFile$2(filePath, "utf8");
  return JSON.parse(content);
}
function writeJsonFile(filePath, content) {
  const data = JSON.stringify(content, null, 2) + `
`;
  return writeFile(filePath, data, { encoding: "utf8" });
}
async function writeFileWithOverwritePrompt(filePath, content, options) {
  const { default: defaultVal, force = !1, ...writeOptions } = options, printablePath = filePath.startsWith(process.cwd()) ? path__default.default.relative(process.cwd(), filePath) : filePath;
  return await fileEqualsData(filePath, content) || !force && await fileExists(filePath) && !await prompt(`File "${printablePath}" already exists. Overwrite?`, {
    type: "confirm",
    default: defaultVal
  }) ? !1 : (await writeFile(filePath, content, writeOptions), !0);
}
async function copyFileWithOverwritePrompt(from, to, flags) {
  const printablePath = to.startsWith(process.cwd()) ? path__default.default.relative(process.cwd(), to) : to;
  return await filesAreEqual(from, to) || (await ensureDirectoryExists(to), !flags.force && await fileExists(to) && !await prompt(`File "${printablePath}" already exists. Overwrite?`, {
    type: "confirm",
    default: !1
  })) ? !1 : (await copyFile(from, to), !0);
}
async function ensureDirectoryExists(filePath) {
  const dirname = path__default.default.dirname(filePath);
  if (await fileExists(dirname))
    return !0;
  await ensureDirectoryExists(dirname), await mkdir(dirname);
}
async function fileEqualsData(filePath, content) {
  const contentHash = crypto__default.default.createHash("sha1").update(content).digest("hex"), remoteHash = await getFileHash(filePath);
  return contentHash === remoteHash;
}
async function filesAreEqual(file1, file2) {
  const [hash1, hash2] = await Promise.all([getFileHash(file1, !1), getFileHash(file2)]);
  return hash1 === hash2;
}
function getFileHash(filePath, allowMissing = !0) {
  return new Promise((resolve, reject) => {
    const hash = crypto__default.default.createHash("sha1"), stream = fs__default.default.createReadStream(filePath);
    stream.on("error", (err) => {
      err.code === "ENOENT" && allowMissing ? resolve(null) : reject(err);
    }), stream.on("end", () => resolve(hash.digest("hex"))), stream.on("data", (chunk) => hash.update(chunk));
  });
}
async function ensureDir(dirPath) {
  try {
    await mkdir(dirPath);
  } catch (err) {
    if (err.code !== "EEXIST")
      throw err;
  }
}
async function isEmptyish(dirPath) {
  const ignoredFiles = [".git", ".gitignore", "license", "readme.md"];
  return (await readdir(dirPath).catch(() => [])).filter((file) => !ignoredFiles.includes(file.toLowerCase())).length === 0;
}
async function readFileContent({
  filename,
  basePath
}) {
  const filepath = path__default.default.normalize(path__default.default.join(basePath, filename));
  try {
    return await readFile$2(filepath, "utf8");
  } catch (err) {
    if (err.code === "ENOENT") {
      cli.log.debug(`No ${filename} file found.`);
      return;
    }
    throw new Error(`Failed to read "${filepath}": ${err.message}`);
  }
}
async function readJson5File({
  filename,
  basePath
}) {
  const content = await readFileContent({ filename, basePath });
  if (content)
    return parseJson5(content, filename);
}
function parseJson5(content, errorKey) {
  try {
    return lib.parse(content);
  } catch (err) {
    throw new Error(`Error parsing "${errorKey}": ${err.message}`);
  }
}
function errorToUndefined(err) {
  if (err instanceof TypeError)
    throw err;
}
const stat = util__default.default.promisify(fs__default.default.stat), readFile$1 = util__default.default.promisify(fs__default.default.readFile), allowedPartProps = ["name", "implements", "path", "description"], disallowedPluginProps = ["api", "project", "plugins", "env"];
async function getPaths(options) {
  const { basePath } = options, manifest = await readManifest(options);
  return manifest.paths ? absolutifyPaths(manifest.paths, basePath) : null;
}
function absolutifyPaths(paths, basePath) {
  const getPath = (relative) => relative ? path__default.default.resolve(path__default.default.join(basePath, relative)) : void 0;
  return paths ? {
    basePath,
    compiled: getPath(paths.compiled),
    source: getPath(paths.source)
  } : { basePath };
}
async function readManifest(options) {
  const { basePath, validate = !0 } = options, manifestPath = path__default.default.normalize(path__default.default.join(basePath, "sanity.json"));
  let content;
  try {
    content = await readFile$1(manifestPath, "utf8");
  } catch (err) {
    throw err.code === "ENOENT" ? new Error(
      `No sanity.json found. sanity.json is required for plugins to function. Use \`${_package.pkg.binname} init\` for a new plugin, or create an empty \`sanity.json\` with an empty object (\`{}\`) for existing ones.`
    ) : new Error(`Failed to read "${manifestPath}": ${err.message}`);
  }
  let parsed;
  try {
    parsed = JSON.parse(content);
  } catch (err) {
    throw new Error(`Error parsing "${manifestPath}": ${err.message}`);
  }
  return validate && await validateManifest(parsed, options), parsed;
}
async function validateManifest(manifest, opts) {
  const options = { isPlugin: !0, ...opts };
  if (!isObject$1(manifest))
    throw new Error("Invalid sanity.json: Root must be an object");
  if (options.isPlugin ? await validatePluginManifest(manifest, options) : await validateProjectManifest(manifest), "root" in manifest && typeof manifest.root != "boolean")
    throw new Error('Invalid sanity.json: "root" property must be a boolean if declared');
  await validateParts(manifest, {
    ...options,
    paths: absolutifyPaths(manifest.paths, options.basePath)
  });
}
function validateProjectManifest(manifest) {
  if ("paths" in manifest)
    throw new Error('Invalid sanity.json: "paths" property has no meaning in a project manifest');
}
async function validatePluginManifest(manifest, options) {
  const disallowed = Object.keys(manifest).filter((key2) => disallowedPluginProps.includes(key2)).map((key2) => `"${key2}"`);
  if (disallowed.length > 0) {
    const plural = disallowed.length > 1 ? "s" : "", joined = disallowed.join(", ");
    throw new Error(
      `Invalid sanity.json: Key${plural} ${joined} ${plural ? "are" : "is"} not allowed in a plugin manifest`
    );
  }
  if (manifest.root)
    throw new Error('Invalid sanity.json: "root" cannot be truthy in a plugin manifest');
  await validatePaths$1(manifest, options);
}
async function validatePaths$1(manifest, options) {
  if (!("paths" in manifest))
    return;
  if (!isObject$1(manifest.paths))
    throw new Error('Invalid sanity.json: "paths" must be an object if declared');
  if (typeof manifest.paths.compiled != "string")
    throw new Error(
      'Invalid sanity.json: "paths" must have a (string) "compiled" property if declared'
    );
  if (typeof manifest.paths.source != "string")
    throw new Error(
      'Invalid sanity.json: "paths" must have a (string) "source" property if declared'
    );
  const sourcePath = path__default.default.resolve(options.basePath, manifest.paths.source);
  let srcStats;
  try {
    srcStats = await stat(sourcePath);
  } catch (err) {
    if (err.code === "ENOENT")
      throw new Error(`sanity.json references "source" path which does not exist: "${sourcePath}"`);
  }
  if (!srcStats?.isDirectory())
    throw new Error(
      `sanity.json references "source" path which is not a directory: "${sourcePath}"`
    );
}
async function validateParts(manifest, options) {
  if (!("parts" in manifest))
    return;
  if (!Array.isArray(manifest.parts))
    throw new Error('Invalid sanity.json: "parts" must be an array if declared');
  let i = 0;
  for (const part of manifest.parts)
    await validatePart(part, i, options), i++;
}
async function validatePart(part, index, options) {
  if (!isObject$1(part))
    throw new Error(`Invalid sanity.json: "parts[${index}]" must be an object`);
  validateAllowedPartKeys(part, index), validatePartStringValues(part, index), validatePartNames(part, index, options), await validatePartFiles(part, index, options);
}
async function validatePartFiles(part, index, options) {
  const { verifyCompiledParts, verifySourceParts, paths } = options;
  if (!part?.path)
    return;
  const ext = path__default.default.extname(part.path);
  if (paths?.source && ext && ext !== ".js" && buildExtensions.includes(ext))
    throw new Error(
      `Invalid sanity.json: Part path has extension which is not applicable after compiling. ${ext} becomes .js after compiling. Specify filename without extension (${path__default.default.basename(
        part.path
      )}) (parts[${index}])`
    );
  if (!verifySourceParts && !verifyCompiledParts)
    return;
  const [srcExists, outDirExists] = await Promise.all([
    hasSourceFile(part.path, paths),
    verifyCompiledParts && hasCompiledFile(part.path, paths)
  ]);
  if (!srcExists)
    throw new Error(
      `Invalid sanity.json: Part path references file that does not exist in source directory (${paths?.source || paths?.basePath}) (parts[${index}])`
    );
  if (verifyCompiledParts && !outDirExists)
    throw new Error(
      `Invalid sanity.json: Part path references file ("${part.path}") that does not exist in compiled directory (${paths?.compiled}) (parts[${index}])`
    );
}
function validatePartNames(part, index, options) {
  const pluginName = options.pluginName ? options.pluginName.replace(/^sanity-plugin-/, "") : "";
  if (!part?.name || !part?.name?.startsWith(`part:${pluginName}/`))
    throw new Error(
      `Invalid sanity.json: "name" must be prefixed with "part:${pluginName}/" - got "${part?.name}" (parts[${index}])`
    );
  if (!part?.implements?.startsWith("part:"))
    throw new Error(
      `Invalid sanity.json: "implements" must be prefixed with "part:" - got "${part?.implements}" (parts[${index}])`
    );
}
function validateAllowedPartKeys(part, index) {
  const disallowed = Object.keys(part).filter((key2) => !allowedPartProps.includes(key2)).map((key2) => `"${key2}"`);
  if (disallowed.length > 0) {
    const plural = disallowed.length > 1 ? "s" : "", joined = disallowed.join(", ");
    throw new Error(
      `Invalid sanity.json: Key${plural} ${joined} ${plural ? "are" : "is"} not allowed in a part declaration (parts[${index}])`
    );
  }
}
function validatePartStringValues(part, index) {
  const nonStrings = Object.keys(part).filter((key2) => typeof part[key2] != "string").map((key2) => `"${key2}"`);
  if (nonStrings.length > 0) {
    const plural = nonStrings.length > 1 ? "s" : "", joined = nonStrings.join(", ");
    throw new Error(
      `Invalid sanity.json: Key${plural} ${joined} should be of type string (parts[${index}])`
    );
  }
}
function isObject$1(obj) {
  return !Array.isArray(obj) && obj !== null && typeof obj == "object";
}
async function hasSanityJson(basePath) {
  const file = await readJsonFile(path__default.default.join(basePath, "sanity.json")).catch(
    errorToUndefined
  );
  return { exists: !!file, isRoot: !!(file && file.root) };
}
async function findStudioV3Config(basePath) {
  const jsFile = "sanity.config.js";
  if (await fileExists(path__default.default.join(basePath, jsFile)))
    return { v3ConfigFile: jsFile };
  const tsFile = "sanity.config.ts";
  return { v3ConfigFile: await fileExists(path__default.default.join(basePath, tsFile)) ? tsFile : void 0 };
}
const lockedDependencies = {
  "styled-components": "^5.2"
};
function resolveLatestVersions(packages) {
  const versions = {};
  for (const pkgName of packages)
    versions[pkgName] = pkgName in lockedDependencies ? lockedDependencies[pkgName] : "latest";
  return pProps__default.default(
    versions,
    async (range, pkgName) => {
      const version = await getLatestVersion__default.default(pkgName, { range });
      if (!version)
        throw new Error(`Found no version for ${pkgName}`);
      return rangeify(version);
    },
    { concurrency: 8 }
  );
}
function rangeify(version) {
  return `^${version}`;
}
const mergedPackages = [
  "@sanity/base",
  "@sanity/core",
  "@sanity/types",
  "@sanity/data-aspects",
  "@sanity/default-layout",
  "@sanity/default-login",
  "@sanity/desk-tool",
  "@sanity/field",
  "@sanity/form-builder",
  "@sanity/initial-value-templates",
  "@sanity/language-filter",
  "@sanity/production-preview",
  "@sanity/react-hooks",
  "@sanity/resolver",
  "@sanity/state-router",
  "@sanity/structure",
  "@sanity/studio-hints"
].sort(), deprecatedDevDeps = [
  "tsdx",
  "sanipack",
  "parcel",
  "@parcel/packager-ts",
  "@parcel/transformer-typescript-types"
], expectedScripts = {
  build: "plugin-kit verify-package --silent && pkg-utils build --strict --check --clean",
  watch: "pkg-utils watch --strict",
  "link-watch": "plugin-kit link-watch",
  prepublishOnly: "npm run build"
};
function filesWithSuffixes(fileBases, suffixes) {
  return fileBases.flatMap((file) => suffixes.map((suffix) => `${file}.${suffix}`));
}
function validateNodeEngine(packageJson) {
  const nodeVersionRange = ">=18";
  if (!packageJson.engines?.node?.startsWith(nodeVersionRange))
    return [
      outdent__default.default`
        Expected package.json to contain engines.node: ">=18" to ensure Studio compatible builds,
        but it was: ${packageJson.engines?.node}

        Please add the following to package.json:

        "engines": {
          "node": "${nodeVersionRange}"
        }`.trimStart()
    ];
}
function validateScripts(packageJson) {
  const errors = [], divergentScripts = Object.entries(expectedScripts).filter(([key2, expectedCommand]) => {
    const command = packageJson.scripts?.[key2];
    return !command || !command.includes(expectedCommand);
  });
  return divergentScripts.length && errors.push(
    outdent__default.default`
      The following script commands did not contain expected defaults: ${divergentScripts.map(([key2]) => key2).join(", ")}

      This checks for that the commands-strings includes these terms.

      Please add the following to your package.json "scripts":

      ${divergentScripts.map(([key2, value]) => `"${key2}": "${value}"`).join(`,
`)}
  `.trimStart()
  ), errors;
}
async function validateTsConfig(ts, options) {
  const { basePath, outDir, tsconfig } = options, errors = [], wrongEntries = Object.entries({
    target: "esnext",
    jsx: "preserve",
    module: "preserve",
    rootDir: ".",
    outDir,
    noEmit: !0
  }).filter(([key2, value]) => {
    let option = ts.options[key2];
    return key2 === "rootDir" && typeof option == "string" && (option = path__default.default.relative(basePath, option) || "."), key2 === "outDir" && typeof option == "string" && (option = path__default.default.relative(basePath, option) || "."), key2 === "target" && option === 99 && (option = "esnext"), key2 === "module" && option === 200 && (option = "preserve"), key2 === "jsx" && option === 1 && (option = "preserve"), typeof value == "string" && typeof option == "string" ? value.toLowerCase() !== option?.toLowerCase() : value !== option;
  });
  if (wrongEntries.length) {
    const expectedOutput = wrongEntries.map(([key2, value]) => `"${key2}": ${typeof value == "string" ? `"${value}"` : value},`).join(`
`);
    errors.push(
      outdent__default.default`
        Recommended ${tsconfig} compilerOptions missing:

        The following fields had unexpected values: [${wrongEntries.map(([key2]) => key2).join(", ")}]
        Expected to find these values:
        ${expectedOutput}

        Please update your ${tsconfig} accordingly.
      `.trimStart()
    );
  }
  return errors;
}
function validatePkgUtilsDependency({ devDependencies }) {
  return devDependencies?.["@sanity/pkg-utils"] ? [] : [
    outdent__default.default`
        package.json does not list @sanity/pkg-utils as a devDependency.
        @sanity/pkg-utils replaced parcel as the recommended build tool in @sanity/plugin-kit 2.0.0

        Please add it by running 'npm install --save-dev @sanity/pkg-utils'.
    `.trimStart()
  ];
}
function validateSanityDependencies(packageJson) {
  const { dependencies, devDependencies, peerDependencies } = packageJson, allDependencies = { ...dependencies, ...devDependencies, ...peerDependencies }, illegalDeps = Object.keys(allDependencies).filter((dep) => mergedPackages.includes(dep)), unique = [...new Set(illegalDeps).values()];
  return unique.length ? [
    outdent__default.default`
        package.json depends on "@sanity/*" packages that have moved into "sanity" package.

        The following dependencies should be replaced with "sanity":
        - ${unique.join(`
- `)}

        Refer to the reference docs to find replacement imports:
        ${cli.urls.refDocs}
    `.trimStart()
  ] : [];
}
function validateDeprecatedDependencies(packageJson) {
  const { dependencies, devDependencies, peerDependencies } = packageJson, allDependencies = { ...dependencies, ...devDependencies, ...peerDependencies }, illegalDeps = Object.keys(allDependencies).filter((dep) => deprecatedDevDeps.includes(dep)), unique = [...new Set(illegalDeps).values()];
  return unique.length ? [
    outdent__default.default`
        package.json contains deprecated dependencies that should be removed:
        - ${unique.join(`
- `)}
    `.trimStart()
  ] : [];
}
async function validateBabelConfig({ basePath }) {
  const filenames = [".babelrc", ...filesWithSuffixes([".babelrc", "babel.config"], ["json", "js", "cjs", "mjs"])], babelFiles = [];
  for (const filename of filenames) {
    const filepath = path__default.default.normalize(path__default.default.join(basePath, filename));
    await fileExists(filepath) && babelFiles.push(filename);
  }
  return babelFiles.length ? [
    outdent__default.default`
        Found babel-config file: [${babelFiles.join(
      ", "
    )}]. When using default @sanity/plugin-kit build command,
        this is probably not needed.

        Delete the file, or disable this check.
      `.trimStart()
  ] : [];
}
async function validateStudioConfig({ basePath }) {
  const suffixes = ["ts", "js", "tsx", "jsx"], filenames = filesWithSuffixes(["sanity.config", "sanity.cli"], suffixes), files = {};
  for (const filename of filenames) {
    const filepath = path__default.default.normalize(path__default.default.join(basePath, filename));
    files[filename] = await fileExists(filepath);
  }
  const sanityJson = await readJson5File({ basePath, filename: "sanity.json" }), hasConfigFile = (fileBase) => filesWithSuffixes([fileBase], suffixes).some((filename) => files[filename]), hasCliConfig = hasConfigFile("sanity.cli"), hasStudioConfig = hasConfigFile("sanity.config"), errors = [];
  if (sanityJson) {
    const info = [
      outdent__default.default`
        Found sanity.json. This file is not used by Sanity Studio V3.

        Please consult the Studio V3 migration guide:
         ${cli.urls.migrationGuideStudio}
        It will detail how to convert sanity.json to sanity.config.ts (or .js) and sanity.cli.ts (or .js) equivalents.
      `.trimStart(),
      sanityJson.plugins?.length && outdent__default.default`
        For V3 versions and alternatives to V2 plugins, please refer to the Sanity Exchange:
        ${cli.urls.sanityExchange}
      `.trimStart()
    ].filter((s) => !!s);
    errors.push(info.join(`

`));
  }
  return hasCliConfig || errors.push(
    outdent__default.default`
        sanity.cli.(${suffixes.join(
      " | "
    )}) missing. Please create a file named sanity.cli.ts with the following content:

        ${chalk__default.default.green(
      outdent__default.default`
        import {createCliConfig} from 'sanity/cli'

        export default createCliConfig({
          api: {
            projectId: '${sanityJson?.api?.projectId ?? "project-id"}',
            dataset: '${sanityJson?.api?.dataset ?? "dataset"}',
          }
        })`
    )}

        Make sure to replace the projectId and dataset fields with your own.

        For more, see ${cli.urls.migrationGuideStudio}
    `.trimStart()
  ), hasStudioConfig || errors.push(
    outdent__default.default`
        sanity.config.(${suffixes.join(
      " | "
    )}) missing. At a minimum sanity.config.ts should contain:

        ${chalk__default.default.green(
      outdent__default.default`
            import { defineConfig } from "sanity"
            import { deskTool } from "sanity/desk"

            export default defineConfig({
              name: "default",

              projectId: '${sanityJson?.api?.projectId ?? "project-id"}',
              dataset: '${sanityJson?.api?.dataset ?? "dataset"}',

              plugins: [
                deskTool(),
              ],

              schema: {
                types: [
                  /* put your v2 schema-types here */
                ],
              },
            })`
    ).trimStart()}

        Make sure to replace the projectId and dataset fields with your own.

        For more, see ${cli.urls.migrationGuideStudio}
    `.trimStart()
  ), errors.length ? [errors.join(`

---

`)] : [];
}
async function validatePluginSanityJson({
  basePath,
  packageJson
}) {
  const sanityJson = await readJson5File({ basePath, filename: "sanity.json" }), expectedDefaults = {
    parts: [
      {
        implements: "part:@sanity/base/sanity-root",
        path: "./v2-incompatible.js"
      }
    ]
  }, hasSinglePart = sanityJson && Object.keys(sanityJson).length === 1 && sanityJson?.parts && sanityJson.parts.length === 1, firstPart = hasSinglePart ? sanityJson?.parts?.[0] : void 0, correctImplements = firstPart?.implements === expectedDefaults.parts[0].implements, pathExists = firstPart?.path && await fileExists(path__default.default.normalize(path__default.default.join(basePath, firstPart.path))), hasDependency = !!packageJson.dependencies?.[cli.incompatiblePluginPackage];
  if (!(sanityJson && hasSinglePart && correctImplements && pathExists && hasDependency)) {
    const errors = [
      sanityJson ? null : "sanity.json does not exist",
      hasSinglePart ? null : 'sanity.json should have exactly one entry in "parts", but did not.',
      correctImplements ? null : `The part should implement ${expectedDefaults.parts[0].implements}, but did not.`,
      firstPart?.path && !pathExists ? `The file in "path", ${firstPart?.path}, does not exist.` : null,
      hasDependency ? null : outdent__default.default`
      package.json should have ${cli.incompatiblePluginPackage} as a dependency, but did not.
        Install it with: npm install --save ${cli.incompatiblePluginPackage}
      `.trimStart()
    ].filter((e) => !!e);
    return [
      outdent__default.default`
        Invalid sanity.json. It is used for compatibility checking in V2 studios:

        - ${errors.join(`
- `)}

        sanity.json will only be used when incorrectly installing a v3 plugin in a v2 Studio.

        This check ensures that sanity.json conforms with the usage section of
        ${cli.urls.incompatiblePlugin}
    `.trimStart()
    ];
  }
  return [];
}
function validatePackageName$1(packageJson) {
  const valid = validateNpmPackageName__default.default(
    packageJson.name
  );
  if (!valid.validForNewPackages)
    return [`Invalid package.json: "name" is invalid: ${valid.errors.join(", ")}`];
  if (!packageJson.name?.startsWith("@") && !packageJson.name?.startsWith("sanity-plugin-"))
    return [
      'Invalid package.json: "name" should be prefixed with "sanity-plugin-" (or scoped - @your-company/plugin-name)'
    ];
}
async function validateSrcIndexFile(basePath) {
  const paths = ["index.js", "index.ts"].map((p) => path__default.default.join("src", p)), allowedIndexFiles = paths.map((file) => path__default.default.join(basePath, file));
  let hasIndex = !1;
  for (const indexFile of allowedIndexFiles)
    hasIndex = hasIndex || await fileExists(indexFile);
  return hasIndex ? [] : [
    outdent__default.default`
      Expected one of [${paths.join(", ")}] to exist.

      @sanity/pkg-utils expects a non-jsx file to be the source entry-point for the plugin.
      If you currently have JSX in your index file, extract it into a separate file and import it.
      `
  ];
}
async function disallowDuplicateConfig({
  basePath,
  pkgJson,
  configKey,
  files
}) {
  const found = [];
  for (const file of files) {
    const filePath = path__default.default.join(basePath, file);
    await fileExists(filePath) && found.push(file);
  }
  return found.length > 1 ? [
    outdent__default.default`
      Found multiple config files that serve the same purpose: [${found.join(", ")}].

      There should be at most one of these files. Delete the rest.
      `
  ] : found.length && pkgJson[configKey] ? [
    outdent__default.default`
      package.json contains ${configKey}, but there also exists a config file that serves the same purpose.
      Config file: ${found.join("")}]

      Either delete the file or remove ${configKey} entry from package.json.
      `
  ] : [];
}
async function disallowDuplicateEslintConfig(basePath, pkgJson) {
  return disallowDuplicateConfig({
    basePath,
    pkgJson,
    configKey: "eslint",
    files: [
      ".eslintrc",
      ".eslintrc.js",
      ".eslintrc.cjs",
      ".eslintrc.yaml",
      ".eslintrc.yml",
      ".eslintrc.json"
    ]
  });
}
async function disallowDuplicatePrettierConfig(basePath, pkgJson) {
  return disallowDuplicateConfig({
    basePath,
    pkgJson,
    configKey: "prettier",
    files: [
      ".prettierrc",
      ".prettierrc.json5",
      ".prettierrc.json",
      ".prettierrc.yaml",
      ".prettierrc.yml",
      ".prettierrc.js",
      ".prettierrc.cjs",
      ".prettier.config,js",
      ".prettier.config.cjs",
      ".prettierrc.toml"
    ]
  });
}
const forcedPackageVersions = {}, forcedDevPackageVersions = forcedPackageVersions, forcedPeerPackageVersions = {
  react: "^18",
  "react-dom": "^18",
  "@types/react": "^18",
  "@types/react-dom": "^18",
  sanity: "^3",
  "styled-components": "^5.2"
}, defaultDependencies = [cli.incompatiblePluginPackage], defaultDevDependencies = [
  "sanity",
  // peer dependencies of `sanity`
  "react",
  "react-dom",
  "styled-components"
], defaultPeerDependencies = ["react", "sanity"], readFile = util__default.default.promisify(fs__default.default.readFile), pathKeys = ["main", "module", "browser", "types"];
async function getPackage(opts) {
  const options = { flags: {}, ...opts };
  validateOptions(options);
  const { basePath, validate = !0 } = options, manifestPath = path__default.default.normalize(path__default.default.join(basePath, "package.json"));
  let content;
  try {
    content = await readFile(manifestPath, "utf8");
  } catch (err) {
    throw err.code === "ENOENT" ? new Error(
      `No package.json found. package.json is required to publish to npm. Use \`${cli.cliName} init\` for a new plugin, or \`npm init\` for an existing one`
    ) : new Error(`Failed to read "${manifestPath}": ${err.message}`);
  }
  let parsed;
  try {
    parsed = JSON.parse(content);
  } catch (err) {
    throw new Error(`Error parsing "${manifestPath}": ${err.message}`);
  }
  if (!isObject(parsed))
    throw new Error("Invalid package.json: Root must be an object");
  return validate && await validatePackage(parsed, options), parsed;
}
async function validatePackage(manifest, opts) {
  validateOptions(opts);
  const options = { isPlugin: !0, ...opts };
  options.isPlugin && await validatePluginPackage(manifest, options), validateLockFiles(options);
}
function validateOptions(opts) {
  const options = opts || {};
  if (!isObject(options))
    throw new Error("Options must be an object");
  if (typeof options.basePath != "string")
    throw new Error('"options.basePath" must be a string (path to plugin base path)');
}
async function validatePluginPackage(manifest, options) {
  validatePackageName(manifest), await validatePaths(manifest, options);
}
function validatePackageName(manifest) {
  if (typeof manifest.name != "string")
    throw new Error('Invalid package.json: "name" must be a string');
  const valid = validateNpmPackageName__default.default(
    manifest.name
  );
  if (!valid.validForNewPackages)
    throw new Error(`Invalid package.json: "name" is invalid: ${valid.errors.join(", ")}`);
  if (manifest.name[0] !== "@" && !manifest.name.startsWith("sanity-plugin-"))
    throw new Error(
      'Invalid package.json: "name" should be prefixed with "sanity-plugin-" (or scoped - @your-company/plugin-name)'
    );
}
async function validatePaths(manifest, options) {
  const paths = await getPaths({
    ...options,
    pluginName: manifest.name ?? "unknown",
    verifySourceParts: !1,
    verifyCompiledParts: !1
  }), abs = (file) => path__default.default.isAbsolute(file) ? file : path__default.default.resolve(path__default.default.join(options.basePath, file)), exists = (file) => fs__default.default.existsSync(abs(file)), willExist = (file) => paths && hasSourceEquivalent(abs(file), paths), withinSourceDir = (file) => paths?.source && abs(file).startsWith(paths.source), withinTargetDir = (file) => paths?.compiled && abs(file).startsWith(paths.compiled);
  for (const key2 of pathKeys) {
    if (!(key2 in manifest))
      continue;
    const manifestValue = manifest[key2];
    if (typeof manifestValue != "string")
      throw new Error(`Invalid package.json: "${key2}" must be a string if defined`);
    if (!options?.flags?.allowSourceTarget && paths && withinSourceDir(manifestValue))
      throw new Error(
        `Invalid package.json: "${key2}" points to file within source (uncompiled) directory. Use --allow-source-target if you really want to do this.`
      );
    if (exists(manifestValue) && paths && withinTargetDir(manifestValue) && !await willExist(manifestValue))
      throw new Error(
        `Invalid package.json: "${key2}" points to file that will not exist after compiling`
      );
    if (!exists(manifestValue) && !await willExist(manifestValue)) {
      if (!paths)
        throw new Error(`Invalid package.json: "${key2}" points to file that does not exist`);
      const inOutDir = paths.compiled && !abs(manifestValue).startsWith(paths.compiled);
      throw new Error(
        inOutDir ? `Invalid package.json: "${key2}" points to file that does not exist, and "paths" is not configured to compile to this location` : `Invalid package.json: "${key2}" points to file that does not exist, and no equivalent is found in source directory`
      );
    }
  }
}
function isObject(obj) {
  return !Array.isArray(obj) && obj !== null && typeof obj == "object";
}
function validateLockFiles(options) {
  const npm = fs__default.default.existsSync(path__default.default.join(options.basePath, "package-lock.json")), yarn = fs__default.default.existsSync(path__default.default.join(options.basePath, "yarn.lock"));
  if (npm && yarn)
    throw new Error("Invalid plugin: contains both package-lock.json and yarn.lock");
}
async function writePackageJson(data, options) {
  const { user, pluginName, license, description, pkg: prevPkg, gitOrigin } = data, {
    outDir,
    peerDependencies: addPeers,
    dependencies: addDeps,
    devDependencies: addDevDeps
  } = options, { flags } = options, prev = prevPkg || {}, usePrettier = flags.prettier !== !1, useEslint = flags.eslint !== !1, useTypescript = flags.eslint !== !1, newDevDependencies = [cli.cliName, "@sanity/pkg-utils"];
  useTypescript && (cli.log.debug("Using TypeScript. Adding to dev dependencies."), newDevDependencies.push("@types/react", "typescript")), usePrettier && (cli.log.debug("Using prettier. Adding to dev dependencies."), newDevDependencies.push("prettier", "prettier-plugin-packagejson")), useEslint && (cli.log.debug("Using eslint. Adding to dev dependencies."), newDevDependencies.push(
    "eslint",
    "eslint-config-sanity",
    "eslint-plugin-react",
    "eslint-plugin-react-hooks"
  ), usePrettier && newDevDependencies.push("eslint-config-prettier", "eslint-plugin-prettier"), useTypescript && newDevDependencies.push("@typescript-eslint/eslint-plugin", "@typescript-eslint/parser")), cli.log.debug("Resolving latest versions for %s", newDevDependencies.join(", "));
  const dependencies = forceDependencyVersions(
    {
      ...prev.dependencies || {},
      ...addDeps || {},
      ...await resolveLatestVersions(defaultDependencies)
    },
    forcedPackageVersions
  ), devDependencies = forceDependencyVersions(
    {
      ...addDevDeps || {},
      ...prev.devDependencies || {},
      ...await resolveLatestVersions([...newDevDependencies, ...defaultDevDependencies])
    },
    forcedDevPackageVersions
  ), peerDependencies = forceDependencyVersions(
    {
      ...prev.peerDependencies || {},
      ...addPeers || {},
      ...await resolveLatestVersions(defaultPeerDependencies)
    },
    forcedPeerPackageVersions
  ), source2 = flags.typescript ? "./src/index.ts" : "./src/index.js", files = [outDir, "sanity.json", "src", "v2-incompatible.js"];
  files.sort();
  const forcedOrder = {
    name: pluginName,
    version: prev.version ?? "1.0.0",
    description: description || "",
    keywords: prev.keywords ?? ["sanity", "sanity-plugin"],
    ...urlsFromOrigin(gitOrigin),
    ...repoFromOrigin(gitOrigin),
    license: license ? license.id : "UNLICENSED",
    author: user?.email ? `${user.name} <${user.email}>` : user?.name,
    sideEffects: !1,
    type: "commonjs",
    exports: {
      ".": {
        source: source2,
        import: `./${outDir}/index.mjs`,
        default: `./${outDir}/index.js`
      },
      "./package.json": "./package.json"
    },
    main: `./${outDir}/index.js`,
    ...flags.typescript ? { types: `./${outDir}/index.d.ts` } : {},
    files,
    scripts: { ...prev.scripts },
    dependencies: sortKeys(dependencies),
    devDependencies: sortKeys(devDependencies),
    peerDependencies: sortKeys(peerDependencies),
    engines: {
      node: ">=18"
    }
  }, manifest = {
    ...forcedOrder,
    // Use already configured values by default (if not otherwise specified)
    ...prev || {},
    // We're de-declaring properties because of key order in package.json
    ...forcedOrder
  }, differs = JSON.stringify(prev) !== JSON.stringify(manifest);
  return cli.log.debug("Does manifest differ? %s", differs ? "yes" : "no"), differs && await writePackageJsonDirect(manifest, options), differs ? manifest : prev;
}
function urlsFromOrigin(gitOrigin) {
  const details = githubUrlToObject__default.default(gitOrigin);
  return details ? {
    homepage: `https://github.com/${details.user}/${details.repo}#readme`,
    bugs: {
      url: `https://github.com/${details.user}/${details.repo}/issues`
    }
  } : {};
}
function repoFromOrigin(gitOrigin) {
  return console.log(gitOrigin), gitOrigin ? {
    repository: {
      type: "git",
      url: gitOrigin
    }
  } : {};
}
function addScript(cmd, existing) {
  return existing && existing.includes(cmd) ? existing : cmd;
}
async function addPackageJsonScripts(manifest, options, updateScripts) {
  const originalScripts = manifest.scripts || {}, scripts = updateScripts({ ...originalScripts }), differs = Object.keys(scripts).some((key2) => scripts[key2] !== originalScripts[key2]);
  return differs && await writePackageJsonDirect({ ...manifest, scripts }, options), differs;
}
async function writePackageJsonDirect(manifest, { basePath }) {
  await writeJsonFile(path__default.default.join(basePath, "package.json"), manifest);
}
async function addBuildScripts(manifest, options) {
  return options.flags.scripts ? addPackageJsonScripts(manifest, options, (scripts) => (scripts.build = addScript(expectedScripts.build, scripts.build), scripts.format = addScript("prettier --write --cache --ignore-unknown .", scripts.format), scripts["link-watch"] = addScript(expectedScripts["link-watch"], scripts["link-watch"]), scripts.lint = addScript("eslint .", scripts.lint), scripts.prepublishOnly = addScript(expectedScripts.prepublishOnly, scripts.prepublishOnly), scripts.watch = addScript(expectedScripts.watch, scripts.watch), scripts)) : !1;
}
function sortKeys(unordered) {
  return Object.keys(unordered).sort().reduce((obj, key2) => (obj[key2] = unordered[key2], obj), {});
}
function forceDependencyVersions(deps, versions = forcedPackageVersions) {
  const entries = Object.entries(deps).map((entry) => {
    const [pkg] = entry, forceVersion = versions[pkg];
    return forceVersion ? [pkg, forceVersion] : entry;
  });
  return Object.fromEntries(entries);
}
exports.addBuildScripts = addBuildScripts;
exports.addPackageJsonScripts = addPackageJsonScripts;
exports.addScript = addScript;
exports.copyFileWithOverwritePrompt = copyFileWithOverwritePrompt;
exports.disallowDuplicateEslintConfig = disallowDuplicateEslintConfig;
exports.disallowDuplicatePrettierConfig = disallowDuplicatePrettierConfig;
exports.ensureDir = ensureDir;
exports.errorToUndefined = errorToUndefined;
exports.fileExists = fileExists;
exports.findStudioV3Config = findStudioV3Config;
exports.forceDependencyVersions = forceDependencyVersions;
exports.forcedDevPackageVersions = forcedDevPackageVersions;
exports.forcedPackageVersions = forcedPackageVersions;
exports.getPackage = getPackage;
exports.hasSanityJson = hasSanityJson;
exports.isEmptyish = isEmptyish;
exports.mergedPackages = mergedPackages;
exports.mkdir = mkdir;
exports.prompt = prompt;
exports.promptForPackageName = promptForPackageName;
exports.promptForRepoOrigin = promptForRepoOrigin;
exports.readFile = readFile$2;
exports.readJsonFile = readJsonFile;
exports.resolveLatestVersions = resolveLatestVersions;
exports.sortKeys = sortKeys;
exports.validateBabelConfig = validateBabelConfig;
exports.validateDeprecatedDependencies = validateDeprecatedDependencies;
exports.validateNodeEngine = validateNodeEngine;
exports.validatePackageName = validatePackageName$1;
exports.validatePkgUtilsDependency = validatePkgUtilsDependency;
exports.validatePluginSanityJson = validatePluginSanityJson;
exports.validateSanityDependencies = validateSanityDependencies;
exports.validateScripts = validateScripts;
exports.validateSrcIndexFile = validateSrcIndexFile;
exports.validateStudioConfig = validateStudioConfig;
exports.validateTsConfig = validateTsConfig;
exports.writeFile = writeFile;
exports.writeFileWithOverwritePrompt = writeFileWithOverwritePrompt;
exports.writePackageJson = writePackageJson;
exports.writePackageJsonDirect = writePackageJsonDirect;
//# sourceMappingURL=package.js.map
