"use strict";
var path = require("path"), meow = require("meow"), pkgUtils = require("@sanity/pkg-utils"), _package = require("./package.js"), cli = require("./cli.js"), ts = require("./ts.js"), outdent = require("outdent"), chalk = require("chalk");
function _interopDefaultCompat(e) {
  return e && typeof e == "object" && "default" in e ? e : { default: e };
}
var path__default = /* @__PURE__ */ _interopDefaultCompat(path), meow__default = /* @__PURE__ */ _interopDefaultCompat(meow), outdent__default = /* @__PURE__ */ _interopDefaultCompat(outdent), chalk__default = /* @__PURE__ */ _interopDefaultCompat(chalk);
async function verifyPackage({ basePath, flags }) {
  let errors = [];
  const packageJson = await _package.getPackage({ basePath, validate: !1 }), verifyConfig = packageJson.sanityPlugin?.verifyPackage || {}, packageConfig = await pkgUtils.loadConfig({ cwd: basePath }), outDir = packageConfig?.dist ?? cli.defaultOutDir, tsconfig = packageConfig?.tsconfig ?? "tsconfig.json", validation = ts.createValidator(verifyConfig, flags, errors), ts$1 = await ts.readTSConfig({ basePath, filename: tsconfig });
  if (await validation("packageName", async () => _package.validatePackageName(packageJson)), await validation("pkg-utils", async () => _package.validatePkgUtilsDependency(packageJson)), await validation("srcIndex", async () => _package.validateSrcIndexFile(basePath)), await validation("scripts", async () => _package.validateScripts(packageJson)), await validation("nodeEngine", async () => _package.validateNodeEngine(packageJson)), await validation(
    "duplicateConfig",
    async () => _package.disallowDuplicateEslintConfig(basePath, packageJson)
  ), await validation(
    "duplicateConfig",
    async () => _package.disallowDuplicatePrettierConfig(basePath, packageJson)
  ), ts$1 && await validation("tsconfig", async () => _package.validateTsConfig(ts$1, { basePath, outDir, tsconfig })), await validation("sanityV2Json", async () => _package.validatePluginSanityJson({ basePath, packageJson })), await validation("babelConfig", async () => _package.validateBabelConfig({ basePath })), await validation("dependencies", async () => _package.validateSanityDependencies(packageJson)), await validation(
    "deprecatedDependencies",
    async () => _package.validateDeprecatedDependencies(packageJson)
  ), await validation("eslintImports", async () => ts.validateImports({ basePath })), errors.length)
    throw new Error(
      outdent__default.default`
        Detected validation issues!
        To make this package Sanity v3 compatible, fix the issues starting from the top, or disable any checks you deem unnecessary.

        These issues assume the package uses @sanity/plugin-kit defaults for development and building.
        Refer to ${cli.urls.pluginReadme} for configuration options.

        More information is available here:
        - Studio migration guide: ${cli.urls.migrationGuideStudio}
        - Plugin migration guide: ${cli.urls.migrationGuidePlugin}
        - Reference documentation: ${cli.urls.refDocs}

        ${chalk__default.default.grey(
        `To fail-fast on first detected issue run:
npx ${cli.cliName} verify-package' --single`
      )}
      `.trimStart()
    );
  await ts.runTscMaybe(verifyConfig, ts$1), cli.log.success(
    outdent__default.default`
    No outstanding upgrade issues detected.

    Suggested next steps:
      - Use plugin-kit to build and develop the plugin according to ${cli.urls.pluginReadme}.
      - Build the plugin and fix any compilation errors
      - Test the plugin using the link-watch command
  `.trim()
  );
}
const description = "Verify that a Sanity plugin package is v3 compatible, and print upgrade steps if not.", help = `
Usage
  $ ${cli.cliName} verify-package [dir] [<args>]

Options
  --single      Enables fail-fast mode: Will only output the first validation that fails.
  --silent      Do not print info and warning messages
  --verbose     Log everything. This option conflicts with --silent
  --version     Output the version number
  --help        Output usage information

Each check will describe how they can be individually disabled.

Examples
  # Verify Sanity plugin package in current directory
  $ ${cli.cliName} verify-package

  # Verify Sanity plugin package in my-plugin directory in silent mode
  $ ${cli.cliName} verify-package my-plugin-directory --silent
`;
function run({ argv }) {
  const cli2 = meow__default.default(help, { flags: ts.verifyFlags, argv, description }), basePath = path__default.default.resolve(cli2.input[0] || process.cwd());
  return verifyPackage({ basePath, flags: cli2.flags });
}
exports.default = run;
//# sourceMappingURL=verify-package.js.map
