"use strict";
var path = require("path"), meow = require("meow"), cli = require("./cli.js"), ts = require("./ts.js"), pkgUtils = require("@sanity/pkg-utils"), _package = require("./package.js"), outdent = require("outdent"), chalk = require("chalk");
function _interopDefaultCompat(e) {
  return e && typeof e == "object" && "default" in e ? e : { default: e };
}
var path__default = /* @__PURE__ */ _interopDefaultCompat(path), meow__default = /* @__PURE__ */ _interopDefaultCompat(meow), outdent__default = /* @__PURE__ */ _interopDefaultCompat(outdent), chalk__default = /* @__PURE__ */ _interopDefaultCompat(chalk);
async function verifyStudio({ basePath, flags }) {
  let errors = [];
  const packageJson = await _package.getPackage({ basePath, validate: !1 }), verifyConfig = packageJson.sanityPlugin?.verifyPackage || {}, tsconfig = (await pkgUtils.loadConfig({ cwd: basePath }))?.tsconfig ?? "tsconfig.json", validation = ts.createValidator(verifyConfig, flags, errors), ts$1 = await ts.readTSConfig({ basePath, filename: tsconfig });
  if (await validation("studioConfig", async () => _package.validateStudioConfig({ basePath })), await validation("dependencies", async () => _package.validateSanityDependencies(packageJson)), await validation("eslintImports", async () => ts.validateImports({ basePath })), errors.length)
    throw new Error(
      outdent__default.default`
        Detected validation issues!
        This Sanity Studio is not completely V3 ready. Fix the issues starting from the top, or disable any checks you deem unnecessary.

        More information is available here:
        - Migration guide: ${cli.urls.migrationGuideStudio}
        - Reference documentation: ${cli.urls.refDocs}

        ${chalk__default.default.grey(
        `To fail-fast on first detected issue run:
npx ${cli.cliName} verify-studio --single`
      )}
      `.trimStart()
    );
  await ts.runTscMaybe(verifyConfig, ts$1), cli.log.success(
    outdent__default.default`
    No outstanding upgrade issues detected. Studio is V3 ready!
  `.trim()
  );
}
const description = "Verify that a Sanity Studio is configured correctly for v3, and print upgrade steps if not.", help = `
Usage
  $ ${cli.cliName} verify-studio [dir] [<args>]

Options
  --single      Enables fail-fast mode: Will only output the first validation that fails.
  --silent      Do not print info and warning messages
  --verbose     Log everything. This option conflicts with --silent
  --version     Output the version number
  --help        Output usage information

Each check will describe how they can be individually disabled.

Examples
  # Verify Sanity Studio in current directory
  $ ${cli.cliName} verify-studio

  # Verify Sanity Studio in my-sanity-studio directory in silent mode
  $ ${cli.cliName} verify-studio my-sanity-studio --silent
`;
function run({ argv }) {
  const cli2 = meow__default.default(help, { flags: ts.verifyFlags, argv, description }), basePath = path__default.default.resolve(cli2.input[0] || process.cwd());
  return verifyStudio({ basePath, flags: cli2.flags });
}
exports.default = run;
//# sourceMappingURL=verify-studio.js.map
