"use strict";
var meow = require("meow"), _package = require("./package2.js"), cli = require("./cli.js");
function _interopDefaultCompat(e) {
  return e && typeof e == "object" && "default" in e ? e : { default: e };
}
var meow__default = /* @__PURE__ */ _interopDefaultCompat(meow);
const description = `Show the installed version of ${_package.pkg.name}`, help = `
Usage
  $ ${_package.pkg.binname} version

Options
  --major  Show only the major version
  --minor  Show only the minor version
  --patch  Show only the patch version

Examples
  $ ${_package.pkg.binname} version
  ${_package.pkg.name} version ${_package.pkg.version}

  $ ${_package.pkg.binname} version --major
  ${_package.pkg.version.split(".")[0]}
`, flags = {
  ...cli.sharedFlags,
  major: {
    type: "boolean",
    default: !1
  },
  minor: {
    type: "boolean",
    default: !1
  },
  patch: {
    type: "boolean",
    default: !1
  }
};
function run({ argv }) {
  const cli$1 = meow__default.default(help, { flags, argv, description }), versionParts = _package.pkg.version.split("."), versionNames = ["major", "minor", "patch"], versionFlags = versionNames.filter((flagName) => cli$1.flags[flagName]), versionFlag = versionFlags[0], numVersionFlags = versionFlags.length;
  if (numVersionFlags === 0) {
    cli.log.msg(`${_package.pkg.name} version ${_package.pkg.version}`);
    return;
  }
  if (numVersionFlags > 1)
    throw new Error(
      "--major, --minor and --patch are mutually exclusive - only one can be used at a time"
    );
  const partIndex = versionNames.indexOf(versionFlag);
  cli.log.msg(versionParts[partIndex]);
}
exports.default = run;
//# sourceMappingURL=version.js.map
